# -*- encoding: utf-8 -*-
'''
@File        :bez.py
@Time        :2021/09/28 11:26:28
@Author        :charles kiko
@Version        :1.0
@Contact        :charles_kiko@163.com
@Desc        :None
'''


import configparser
import os
import re
import famCircle
import numpy as np
import pandas as pd
from Bio import Seq, SeqIO, SeqRecord
import codecs
from tqdm import trange
import matplotlib.pyplot as plt
# Bezier functions

def read_lens(lens):
    chr_dic = {}
    chr_list = []
    for i in open(lens, 'r', encoding='utf-8'):
        if i[0] != '#' or i[0] != '\n':
            line = i.strip('\n').split()
            dic0 = {}
            dic0['end'] = int(line[1])
            dic0['order'] = int(line[2])
            chr_dic[line[0]] = dic0
            chr_list.append(line[0])
    return chr_dic,chr_list

def read_gff(gff):
    gene = {}
    gff = open(gff, 'r', encoding='utf-8').readlines()
    for j in range(len(gff)):
        i = gff[j]
        if i[0] != '#' or i[0] != '\n':
            line = i.strip('\n').split()
            dic0 = {}
            dic0['chr'] = line[0]
            dic0['end'] = int(line[2])
            dic0['order'] = int(line[5])
            gene[line[1]] = dic0
    return gene

def blasts():
    file = open(sys.argv[1],'r').readlines()
    for i in trange(len(file)):
        line = file[i]
        lt=line.strip("\n").split("\t")
        if (str(lt[0]) not in blast):
            blast.append(str(lt[0]))
            number=10
        else:
            if number != 0:
                f.write(line)
                number -= 1
            if number == 0:
                continue


def read_blast0(blast0):
    pair_dic = {}
    one = []
    list0 = []
    blast = open(blast0, 'r', encoding='utf-8').readlines()
    for j in trange(len(blast)):
        i = blast[j]
        if i[0] != '#' or i[0] != '\n':
            line = i.strip('\n').split()
            if line[0] not in one:
                if len(list0) != 0:
                    pair_dic[line[0]] = list0
                    list0 = []
                one.append(line[0])
                list0.append(line[1])
            else:
                list0.append(line[1])
    if len(list0) != 0:
        pair_dic[line[0]] = list0
        list0 = []
    return pair_dic

def read_ks0(file0):
    ks = {}
    file = open(file0, 'r', encoding='utf-8').readlines()
    for j in trange(len(file)):
        i = file[j]
        if i[0] == '#' or i[0] == '\n':
            continue
        elif 'id1' in i:
            continue
        else:
            line = i.strip('\n').split()
            if len(line) <= 4:
                continue
            ks[str(line[0]) + '_' + str(line[1])] = float(line[3])
    return ks

def read_family(file):
    family = []
    for i in open(file, 'r', encoding='utf-8'):
        if i[0] != '#' or i[0] != '\n':
            line = i.strip('\n').split()
            if line[0] not in family:
                family.append(line[0])
    return family

def getRGB(dWave,maxPix=1,gamma=1):
    #dWave为波长；maxPix为最大值；gamma为调教参数
    waveArea = [380,440,490,510,580,645,780]
    minusWave = [0,440,440,510,510,645,780]
    deltWave = [1,60,50,20,70,65,35]
    for p in range(len(waveArea)):
        if dWave<waveArea[p]:
            break
    pVar = abs(minusWave[p]-dWave)/deltWave[p]
    rgbs = [[0,0,0],[pVar,0,1],[0,pVar,1],[0,1,pVar],
            [pVar,1,0],[1,pVar,0],[1,0,0],[0,0,0]]
    #在光谱边缘处颜色变暗
    if (dWave>=380) & (dWave<420):
        alpha = 0.3+0.7*(dWave-380)/(420-380)
    elif (dWave>=420) & (dWave<701):
        alpha = 1.0
    elif (dWave>=701) & (dWave<780):
        alpha = 0.3+0.7*(780-dWave)/(780-700)
    else:
        alpha = 0       #非可见区
    return [maxPix*(c*alpha)**gamma for c in rgbs[p]]

def return_col(ks,min_,max_):
    # ks越小波长越长颜色越偏红，紫短红长
    wavelength = 760-(((ks-min_)/(max_-min_))*(760-400))
    return getRGB(wavelength)

def drawSpec(min_,max_,gs):#min_,max_
    ax1 = plt.subplot(gs)
    #波长越大ks越小
    pic = np.zeros([10,360,3])
    rgb = [getRGB(d) for d in range(400,760)]
    pic = pic+rgb
    pic0 = np.zeros([360,10,3])
    for i in range(10):
        for j in range(360):
            pic0[j][i] = pic[i][j]
    plt.imshow(pic0)
    plt.yticks(range(0,360,50),[str(format(max_ - i*((max_-min_)/8), '.2f')) for i in range(8)])      #y坐标轴ks越小颜色越红
    plt.xticks([])

def readname(name,chrolist):
    name = name.lower()
    chrolist = chrolist.lower()
    if ('^' in name):
        return name
    elif ('g' in name):
        if 'g' not in chrolist:
            name = name.replace('g', '^')
        else :
            lt = name.split('g')
            name = ''
            for i in range(len(lt)):
                if i < len(lt) - 2:
                    name = name + str(lt[i]) + 'g'
                elif i == len(lt) - 2:
                    name = name + str(lt[-2])
                else:
                    name = name + "^" + str(lt[-1])
        return name

def config():
    conf = configparser.ConfigParser()
    conf.read(os.path.join(famCircle.__path__[0], 'conf.ini'))
    return conf.items('ini')

def load_conf(file, section):
    conf = configparser.ConfigParser()
    conf.read(file)
    return conf.items(section)

def read_famCircle(file,chrolist):
    f = open(file,'r', encoding='utf-8')
    pair_list = []
    for line in f:
        if line[0] == '#':
            continue
        else:
            lt = line.strip('\n').split()
            genepair = ''
            gene1 = readname(str(lt[0]),chrolist)
            gene2 = readname(str(lt[1]),chrolist)
            if gene1 < gene2:
               genepair = gene1 + " " + gene2
            else:
               genepair = gene2 + " " + gene1
            pair_list.append(genepair)
    return pair_list

def read_coliearity0(file,chrolist):
    f = open(file,'r', encoding='utf-8')
    pair_list = []
    for line in f:
        if line[0] == '#':
            continue
        else:
            lt = line.strip('\n').split()
            genepair = ''
            gene1 = readname(str(lt[0]),chrolist)
            gene2 = readname(str(lt[2]),chrolist)
            if gene1 < gene2:
               genepair = gene1 + " " + gene2
            else:
               genepair = gene2 + " " + gene1
            pair_list.append(genepair)
    return pair_list

def read_blast(file,chrolist):
    f = open(file,'r', encoding='utf-8')
    pair_list = []
    for line in f:
        if line[0] == '#':
            continue
        else:
            lt = line.strip('\n').split()
            genepair = ''
            gene1 = readname(str(lt[0]),chrolist)
            gene2 = readname(str(lt[1]),chrolist)
            if gene1 < gene2:
               genepair = gene1 + " " + gene2
            else:
               genepair = gene2 + " " + gene1
            pair_list.append(genepair)
    return pair_list

def calculate_coef(p0, p1, p2, p3):
    c = 3*(p1 - p0)
    b = 3*(p2 - p1) -c
    a = p3 - p0 - c - b
    return c, b, a

def Bezier(plist, t):
    # p0 : origin, p1, p2 :control, p3: destination
    p0, p1, p2, p3 = plist
    # calculates the coefficient values
    c, b, a = calculate_coef(p0, p1, p2, p3)
    tsquared = t**2
    tcubic = tsquared*t
    return a*tcubic + b*tsquared + c*t + p0

def gene_length(gfffile):
    # 读取基因长度，得到平均长度和最小最大长度
    f = open(gfffile,'r', encoding='utf-8')
    genelength = {}
    for row in f:
        if row[0] != '\n' and row[0] != '#':
            row = row.strip('\n').split('\t')
            if str(row[1]) in genelength.keys():
                continue
            if 'chr' == str(row[1])[3:6]:
                continue
            length = abs(int(row[3]) - int(row[2]))
            genelength[str(row[1])] = length
    f.close()
    lt = []
    for i in genelength.values():
        lt.append(i)
    pj = sum(lt)/len(lt)
    return pj

def gene_length_order(lensfile):
    # 读取基因长度，得到平均长度和最小最大长度
    f = open(lensfile,'r', encoding='utf-8')
    gene_sum = 0
    gene_index = 0
    for row in f:
        if row[0] != '\n' and row[0] != '#':
            row = row.strip('\n').split('\t')
            gene_sum = gene_sum + int(row[2])
            gene_index = gene_index + int(row[1])
    f.close()
    pj = gene_index/gene_sum
    return pj

def cds_to_pep(cds_file, pep_file, fmt='fasta'):
    records = list(SeqIO.parse(cds_file, fmt))
    for k in records:
        k.seq = k.seq.translate()
    SeqIO.write(records, pep_file, 'fasta')
    return True

def read_colinearscan(file):
    data, b, flag, num = [], [], 0, 1
    with open(file) as f:
        for line in f.readlines():
            line = line.strip()
            if re.match(r"the", line):
                num = re.search('\d+', line).group()
                b = []
                flag = 1
                continue
            if re.match(r"\>LOCALE", line):
                flag = 0
                p = re.split(':', line)
                if len(b) > 0:
                    data.append([num, b, p[1]])
                b = []
                continue
            if flag == 1:
                a = re.split(r"\s", line)
                b.append(a)
    return data


def read_mcscanx(fn):
    f1 = open(fn)
    data, b = [], []
    flag, num = 0, 0
    for line in f1.readlines():
        line = line.strip()
        if re.match(r"## Alignment", line):
            flag = 1
            if len(b) == 0:
                arr = re.findall(r"[\d+\.]+", line)[0]
                continue
            data.append([num, b, 0])
            b = []
            num = re.findall(r"\d+", line)[0]
            continue
        if flag == 0:
            continue
        a = re.split(r"\:", line)
        c = re.split(r"\s+", a[1])
        b.append([c[1], c[1], c[2], c[2]])
    data.append([num, b, 0])
    return data


def read_jcvi(fn):
    f1 = open(fn)
    data, b = [], []
    num = 1
    for line in f1.readlines():
        line = line.strip()
        if re.match(r"###", line):
            if len(b) == 0:
                continue
            data.append([num, b, 0])
            b = []
            num += 1
            continue
        a = re.split(r"\t", line)
        b.append([a[0], a[0], a[1], a[1]])
    data.append([num, b, 0])
    return data


def read_coliearity(fn):
    f1 = open(fn)
    data, b = [], []
    flag, num = 0, 0
    for line in f1.readlines():
        line = line.strip()
        if re.match(r"# Alignment", line):
            flag = 1
            if len(b) == 0:
                arr = re.findall('[\.\d+]+', line)
                continue
            data.append([arr[0], b, arr[2]])
            b = []
            arr = re.findall('[\.\d+]+', line)
            continue
        if flag == 0:
            continue
        b.append(re.split(r"\s", line))
    data.append([arr[0], b, arr[2]])
    return data

def read_ks(file, col):
    ks = pd.read_csv(file, sep='\t')
    ks.drop_duplicates(subset=['id1', 'id2'], keep='first', inplace=True)
    ks[col] = ks[col].astype(float)
    ks = ks[ks[col] >= 0]
    ks.index = ks['id1']+','+ks['id2']
    return ks[col]



# -*- encoding: utf-8 -*-
'''
@File        :tamdem.py
@Time        :2021/09/28 11:17:18
@Author        :charles kiko
@Version        :1.0
@Contact        :charles_kiko@163.com
@Desc        :放射型圈图
'''

import re
import os
import sys
import gc
from math import *
import csv
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
from matplotlib.patches import *
from pylab import *
from famCircle.bez import *
from matplotlib import gridspec


class WGD_family():
    def __init__(self, options):
    ##### circle parameters
        self.GAP_RATIO = 5 #gaps between chromosome circle, chr:gap = 4: 1
        self.radius = 0.33
        self.block= 0.008 #block scize
        self.blockthick = 0.007 #0.006
        self.start_list = {}
        self.class1 = True
        for k, v in options:
            setattr(self, str(k), v)
            print(k, ' = ', v)

    def ksrun(self):
        lens,chr_list = read_lens(self.lens)# 染色体字典
        gff = read_gff(self.gff)# 基因字典
        blast = self.readblast(gff)
        ks = read_ks0(self.ks)# ks字典
        family = read_family(self.genefamily)# 家族列表

        for i in gff:
            if gff[i]['chr'] not in lens.keys():
                gff = gff.pop(i)
        for i in family:# 去除不在染色体上的家族成员
            if i not in gff.keys():
                family.remove(i)
        return lens,chr_list,gff,blast,ks,family


    def readblast(self,gff):
        one_gene = []
        alphagenepairs = open(self.genepairs, 'r', encoding='utf-8')
        if self.file_type == 'famCircle':
            for row in alphagenepairs:
                if (row[0] == '\n'):
                    continue
                elif (row[0] == '#'):
                    continue
                else:
                    lt = row.strip('\n').split()
                    id1, id2 = str(lt[0]),str(lt[1])
                    if (id1 not in gff.keys() or id2 not in gff.keys()):
                        continue
                    one_gene.append([id1,id2])
        elif self.file_type == 'WGDI':
            for row in alphagenepairs:
                if (row[0] == '\n'):
                    continue
                elif (row[0] == '#'):
                    continue
                else:
                    lt = row.strip('\n').split()
                    id1, id2 = str(lt[0]),str(lt[2])
                    if (id1 not in gff.keys() or id2 not in gff.keys()):
                        continue
                    one_gene.append([id1,id2])
        elif self.file_type == 'ColinearScan':
            for row in alphagenepairs:
                if (row[0] == '\n' and row[0] == '+'):
                    continue
                elif (row[:3] == 'the'):
                    continue
                else:
                    lt = row.strip('\n').split()
                    id1, id2 = str(lt[0]),str(lt[2])
                    if (id1 not in gff.keys() or id2 not in gff.keys()):
                        continue
                    one_gene.append([id1,id2])
        elif self.file_type == 'MCScanX':
            for row in alphagenepairs:
                if (row[0] == '\n'):
                    continue
                elif(row[:12] == '## Alignment'):
                    continue
                elif ('#' not in row):
                    lt = row.strip('\n').split()
                    # print(lt)
                    id1, id2 = str(lt[2]),str(lt[3])
                    if (id1 not in gff.keys() or id2 not in gff.keys()):
                        continue
                    one_gene.append([id1,id2])
        else:
            print('genepairsfile_type error: File Format not recognized!')
            exit()
        # self.colornum = len(chrnum)
        alphagenepairs.close()
        blastlist = sorted(one_gene,key=(lambda x:x[0]))
        # print(one_gene)
        blast = {}
        for i in one_gene:
            if i[0] not in blast.keys():
                lt = []
                lt.append(i[1])
                blast[i[0]] = lt
            else:
                blast[i[0]].append(i[1])
        # print(blast)
        return blast

    def rad_to_coord(self, angle, radius):
        return radius*cos(angle), radius*sin(angle)

    def to_deg(self, bp, total):
        # from basepair return as degree
        return bp*360./total

    def to_radian(self, bp, total):
        # from basepair return as radian
        # print ("to_radian", bp, total)
        return radians(bp*360./total)

    def plot_arc(self, start, stop, radius):
        # start, stop measured in radian 染色体
        #print start, stop
        t = arange(start, stop, pi/720.)
        x, y = radius*cos(t), radius*sin(t)
        plot(x, y, "k-",alpha=.5)

    def plot_cap(self, angle, clockwise):
        radius=self.sm_radius
        # angle measured in radian, clockwise is boolean 鸭舌
        if clockwise: 
            t = arange(angle, angle+pi, pi/30.)
        else: 
            t = arange(angle, angle-pi, -pi/30.)
        x, y = radius*cos(t), radius*sin(t)
        middle_r = (self.radius_a+self.radius_b)/2
        x, y = x + middle_r*cos(angle), y + middle_r*sin(angle)
        plot(x, y, "k-", alpha=.5)

    def plot_arc_block(self, start, radius):# block
        t = arange(start, start+self.block, pi/720.)
        # print(start, start+self.block)
        x,y = radius * cos(t), radius*sin(t)
        x1, y1 = (radius-self.blockthick) * cos(t), (radius-self.blockthick) * sin(t)
        plot(x, y, "b-", linewidth=3, alpha=0.5)

    def zj(self,lens,chr_list):
        fullchrolen = sum([lens[i]['end'] for i in lens.keys()])
        fullgene = sum([lens[i]['order'] for i in lens.keys()])
        gene_average = int(fullchrolen/fullgene)
        chr_number = len(lens.keys()) # total number of chromosomes
        GAP = fullchrolen/self.GAP_RATIO/chr_number # gap size in base pair
        total_size = fullchrolen + chr_number * GAP # base pairs 
        for i in chr_list:
            if i == chr_list[0]:
                self.start_list[i] = 0
            else:
                self.start_list[i] = self.start_list[chr_list[chr_list.index(i)-1]] + lens[chr_list[chr_list.index(i)-1]]['end'] + GAP
        return total_size,gene_average

    def transform_deg(self, ch, pos, total_size):
        return self.to_deg(pos + self.start_list[ch], total_size)

    def transform_pt(self, ch, pos, r, total_size):
        # convert chromosome position to axis coords
        rad = self.to_radian(pos + self.start_list[ch], total_size)
        return r*cos(rad), r*sin(rad)

    def transform_pt1(self,ch, pos, r, total_size):
        # convert chromosome position to axis coords
        # print("transform", ch, pos, r)
    #    print "startlist", self.start_list[ch]
        rad = self.to_radian(pos + self.start_list[ch], total_size)
        return r*cos(rad), r*sin(rad),rad

    def transform_pt2(self, rad, r):
        return r*cos(rad), r*sin(rad)

    def plot_bez_inner(self, p1, p2, cl, total_size,lw0):
    #    print "inner"
        a, b, c = p1
        ex1x, ex1y = self.transform_pt(a, b, c, total_size)
        a, b, c = p2
        ex2x, ex2y = self.transform_pt(a, b, c, total_size)
        # Bezier ratio, controls curve, lower ratio => closer to center
        ratio = .5
        x = [ex1x, ex1x*ratio, ex2x*ratio, ex2x]
        y = [ex1y, ex1y*ratio, ex2y*ratio, ex2y]
        step = .01
        t = arange(0, 1+step, step)
        xt = Bezier(x, t)# 贝塞尔曲线
        yt = Bezier(y, t)
        plot(xt, yt, '-', color=cl, lw=lw0, alpha=0.3)#alpha 透明度

    def plot_bez_Ks2(self, rad1, r1, rad2, r2, col, ratio):
    #    print "bez Ks 2"
        # print('rad1, r1',rad1, r1)
        ex1x, ex1y = self.transform_pt2(rad1, r1)
        ex2x, ex2y = self.transform_pt2(rad2, r2)
        # ratio = -0.7#0.5
        sita = pi / 2
        if ex1x != ex2x:
            sita = atan((ex2y-ex1y)/(ex2x-ex1x))
        d = sqrt((ex2x-ex1x)**2+(ex2y-ex1y)**2)
        L = d * ratio
        P1x = ex1x + L*sin(sita)
        P1y = ex1y - L*cos(sita)
        P2x = ex2x + L*sin(sita)
        P2y = ex2y - L*cos(sita)
        step = .01
        t = arange(0, 1+step, step)
        x=[ex1x, P1x, P2x, ex2x]
        y=[ex1y, P1y, P2y, ex2y]
        # print('x,y,t',x,y,t)
        xt = Bezier(x,t)
        yt = Bezier(y,t)
        plot(xt, yt, '-', color = col, lw = 0.7)#0.1

    def run(self):
        self.radius_a =  float(self.radius)
        self.radius_b = self.radius_a + 0.005#.33, .335   # 半径控制参数 
        self.sm_radius=(self.radius_b-self.radius_a)/2 #telomere capping
        if (os.path.exists(self.outfile)):
            os.remove(self.outfile)
        lens,chr_list,gff,blast,ks,family = self.ksrun()
        total_size,gene_average = self.zj(lens,chr_list)
        stop_list = {}

        fig = plt.figure(figsize=(12, 10))
        gs = gridspec.GridSpec(1, 2, width_ratios=[10, 1])
        gs0 = gridspec.GridSpec(1, 2)
        plt.subplot(gs[0])

        # for id1 in blast.keys():
        #     if id1 in gff.keys():
        #         for id2 in blast[id1]:
        #             if id2 in gff.keys():
        #                 pos1,pos2 = gff[id1]['end'],gff[id2]['end']
        #                 chro1,chro2 = gff[id1]['chr'],gff[id2]['chr']
        #                 self.plot_bez_inner((chro1, pos1, self.radius_a*0.95), (chro2, pos2, self.radius_a*0.95), "#00a381", total_size,0.8)#绘制内部共线性

        for i in self.start_list.keys():
            stop_list[i] = self.start_list[i] + lens[i]['end']
        for i in chr_list:
            start,stop = self.start_list[i],stop_list[i]
            start, stop = self.to_radian(start, total_size), self.to_radian(stop, total_size)
            # shaft
            self.plot_arc(start, stop, self.radius_a)
            self.plot_arc(start, stop, self.radius_b)
            # telemere capping
            clockwise=False
            self.plot_cap(start, clockwise)
            clockwise=True
            # print(start)
            self.plot_cap(stop, clockwise)
            label_x, label_y = self.rad_to_coord((start+stop)/2, self.radius_b*1.07)#1.2
            #print label_x, label_y
            text(label_x, label_y, i, horizontalalignment="center", verticalalignment="center", fontsize = 7, color = 'black')

        # 绘制关系，并查询WGD_family
        WGD_family_list = []
        outfile = open(self.outfile, 'w', encoding='utf-8')
        out_list = []
        put_ = []
        for id1 in blast.keys():
            if id1 in gff.keys():
                for id2 in blast[id1]:
                    if id2 in gff.keys():
                        pos1,pos2 = gff[id1]['end'],gff[id2]['end']
                        chro1,chro2 = gff[id1]['chr'],gff[id2]['chr']
                        if id1 in family and id2 in family:
                            put_.append([id1,id2])
                            if id1 not in out_list:
                                outfile.write(str(id1) + '\n')
                                out_list.append(id1)
                                outfile.write(str(id2) + '\n')
                                out_list.append(id2)
                        # elif id1 not in family and id2 in family:
                        #     put_.append([id1,id2])
                        #     if id2 not in out_list:
                        #         outfile.write(str(id2) + '\n')
                        #         out_list.append(id2)
                        else:
                            posi1 = gff[id1]['end'] + self.start_list[gff[id1]['chr']]
                            start1 = self.to_radian(posi1, total_size)
                            posi2 = gff[id2]['end'] + self.start_list[gff[id2]['chr']]
                            start2 = self.to_radian(posi2, total_size)
                            self.plot_bez_inner((chro1, pos1, self.radius_a*0.95), (chro2, pos2, self.radius_a*0.95), "#e9dfe5", total_size,2)#绘制内部共线性
        outfile.close()
        out_other = open(self.outother, 'w', encoding='utf-8')
        for i in family:
            if i not in out_list:
                out_other.write(str(i) + '\n')
        out_other.close()
        # print(put_)
        for i in put_:
            id1,id2 = i[0],i[1]
            pos1,pos2 = gff[id1]['end'],gff[id2]['end']
            chro1,chro2 = gff[id1]['chr'],gff[id2]['chr']
            posi1 = gff[id1]['end'] + self.start_list[gff[id1]['chr']]
            start1 = self.to_radian(posi1, total_size)
            posi2 = gff[id2]['end'] + self.start_list[gff[id2]['chr']]
            start2 = self.to_radian(posi2, total_size)
            if str(id1)+'_'+str(id2) in ks.keys():
                ks_v = ks[str(id1)+'_'+str(id2)]
                col = return_col(ks_v,float(self.ks_concern.split(',')[0]),float(self.ks_concern.split(',')[1]))
            elif str(id2)+'_'+str(id1) in ks.keys():
                ks_v = ks[str(id2)+'_'+str(id1)]
                col = return_col(ks_v,float(self.ks_concern.split(',')[0]),float(self.ks_concern.split(',')[1]))
            else:
                col = 'black'
            self.plot_bez_inner((chro1, pos1, self.radius_a*0.98), (chro2, pos2, self.radius_a*0.98), col, total_size,2)#绘制内部共线性
        plt.axis('off')
        drawSpec(float(self.ks_concern.split(',')[0]),float(self.ks_concern.split(',')[1]),gs[1])
        savefig(self.savefile, dpi=1000)
        sys.exit(0)



# -*- encoding: utf-8 -*-
'''
@File        :screen.py
@Time        :2021/09/28 09:03:54
@Author        :charles kiko
@Version        :1.0
@Contact        :charles_kiko@163.com
@Desc        :分染色体展示家族分布折线图
'''


from famCircle.bez import *
import os
import sys
from pylab import *

class screen():
    def __init__(self, options):
        # self.color = ['red', 'fuchsia', 'aquamarine', 'orangered', 'violet', 'lime', 'chocolate', 'blueviolet', 'limegreen', 'orange', 'royalblue', 'gold', 'dodgerblue', 'yellow', 'cyan', 'yellowgreen', 'teal', 'palegreen']
        self.marker = ['.','o','^','v','<','>','s','+','x']
        self.color = ['red', 'fuchsia', 'aquamarine', 'orangered', 'violet', 'lime', 'chocolate', 'blueviolet', 'limegreen', 'orange', 'royalblue', 'gold', 'dodgerblue', 'yellow', 'cyan', 'yellowgreen', 'teal', 'palegreen']
        self.ls = ['-','-','-']
        self.series = "25"
        self.position = 'order'
        for k, v in options:
            setattr(self, str(k), v)
            print(k, ' = ', v)

    def run_plot(self,x_axis_data,y_axis_data,name,tup,domain):
        linestyle = str(tup[0])
        marker = str(tup[1])
        color = str(tup[2])
        # plot中参数的含义分别是横轴值，纵轴值，线的形状，颜色，透明度,线的宽度和标签
        plt.plot(x_axis_data, y_axis_data, marker = marker, markersize = 0.5, linestyle = linestyle, color = color, alpha = 0.5, linewidth = 1, label = domain)
        # 显示标签，如果不加这句，即使在plot中加了label='一些数字'的参数，最终还是不会显示标签
        plt.legend(bbox_to_anchor=(1.05, 1), loc=2, borderaxespad=0)
        plt.xlabel('chr')
        plt.ylabel('gene')

    def readgf(self,file):
        f = open(self.domainpath + '/' + file, 'r', encoding='utf-8')
        genef = []
        for row in f:
            if row[0] != "#" or row != '\n':
                row = row.strip('\n').split()
                name = readname(str(row[0]),self.chro_name)
                if name not in genef:
                    genef.append(name)
        for i in genef:
            if i == None:
                genef.remove(None)
        return genef

    def readgff(self):# 基因开始位点 字典嵌套
        f = open(self.gff,'r', encoding='utf-8')
        genechrs = {}
        gene = {}
        genechr = []
        a = 0
        for row in f:
            if row[0] != '\n' and row[0] != '#':
                row = row.strip('\n').split('\t')
                name = readname(str(row[1]),self.chro_name)
                if name in gene.keys():
                    continue
                if str(row[0]) not in genechr:
                    if a == 1:
                        genechrs[str(genechr[-1])] = gene
                        gene = {}
                        a = 0
                    genechr.append(str(row[0]))
                    gene[name] = str(row[2])
                else:
                    gene[name] = str(row[2])
                    a = 1
        if a == 1:
            genechrs[str(genechr[-1])] = gene
        f.close()
        # print(genechrs)
        return genechrs

    def readlens(self):# 基因开始位点
        f = open(self.lens,'r', encoding='utf-8')
        chrs = {}
        for row in f:
            if row[0] != '\n' and row[0] != '#':
                row = row.strip('\n').split('\t')
                chrs[str(row[0])] = str(row[1])
        f.close()
        return chrs

    def numb(self,i):
        ls = self.ls[i % 3]
        marker = self.marker[i % 9]
        color = self.color[i % 18]
        return ls,marker,color

    def chrlist(self,chrs,length):
        newchrlength = {}
        for chr0 in chrs.keys():
            newlength = int(chrs[chr0]) + length * (int(self.series) - ((int(chrs[chr0]) \
                + (length - (int(chrs[chr0]) % length))) % int(self.series)))
            jishu = newlength / (length * int(self.series))# 级数
            newchrlength[chr0] = jishu
        return newchrlength

    def run(self):
        if os.path.isdir(self.outpath):
            pass
        else:
            d = os.mkdir(self.outpath)
        length = gene_length(self.gff)
        gene = self.readgff()
        chrs = self.readlens()
        name = os.listdir(self.domainpath)
        domains = {}
        for i in range(len(name)):

            domain = name[i]
            # print(domain)
            nm = str(str(domain).split('.')[0])
            # print(nm)
            domains[nm] = self.readgf(domain) + [self.numb(i)]
        data = {}
        for key in domains.keys():# 遍历结构域
            # print(key)
            chrx = {}
            
            sx = domains[key][-1]
            lt = domains[key][:-1]
            # print(key)
            # print(lt)
            chrx['att'] = sx
            for j in range(len(lt) - 1):# 遍历基因
                jishu = {}
                i = lt[j]
                chrg = str(str(i).split('^')[0])
                start = gene[chrg][i]
                
                jishug = int(int(start)/(int(self.series) * length))
                # print(chrg,jishug)
                if chrg not in chrx.keys():
                    jishu[jishug] = 1
                    chrx[chrg] = jishu
                else:
                    if jishug in chrx[chrg].keys():
                        chrx[chrg][jishug] += 1
                    else:
                        chrx[chrg][jishug] = 1
            for i in list(chrx.keys()):
                # print(type(chrx[i]))
                try:
                    length0 = max(list(chrx[i].keys()))
                    # print(length0)
                    for j in range(length0 + 1):
                        if j in list(chrx[i].keys()):
                            pass
                        else:
                            chrx[i][j] = 0
                except:
                    pass
                
            # print("#####################################")
            data[key] = chrx
            chrx = {}
        chry = []
        for domain in data.keys():
            for ch in data[domain].keys():
                if str(ch) == 'att':
                    continue
                else:
                    chry.append(ch)
            break
        domains = list(data.keys())
        # print(data['PF05659_seed']['ath4'])
        # print(data['PF05659_seed']['ath3'])
        # print(domains)
        for i in chry:
            for j in domains:
                tup = data[j]['att']
                name = str(i) + '_' + str(j) + '.png'
                x = sorted(data[j][i].keys())
                # x = list(data[domain][ch].keys())
                y = []
                for o in x:
                    y.append(data[j][i][o])
                self.run_plot(x,y,name,tup,j)
            plt.title(i)
            plt.savefig(self.outpath + '/' +str(i),dpi = 1000,bbox_inches='tight')  # 保存该图片
            plt.cla()
        sys.exit(0)



# -*- encoding: utf-8 -*-
'''
@File        :Ks.py
@Time        :2021/09/28 11:20:37
@Author        :charles kiko
@Version        :1.0
@Contact        :charles_kiko@163.com
@Desc        :计算KS
'''


import os
import re
import sys
from io import StringIO

import numpy as np
import pandas as pd
from Bio import AlignIO, SeqIO
from Bio.Align.Applications import MafftCommandline, MuscleCommandline
from Bio.Phylo.PAML import yn00

import famCircle.bez as bez

import time

import copy
import multiprocessing # Step I : 导入模块
from multiprocessing import cpu_count#读取CPU核心数用于匹配线程数

class Ks():
    def __init__(self, options):
        bez_conf = bez.config()
        self.pair_pep_file = 'pair.pep'
        self.pair_cds_file = 'pair.cds'
        self.prot_align_file = 'prot.aln'
        self.mrtrans = 'pair.mrtrans'
        self.pair_yn = 'pair.yn'
        self.cds_file = 'cds'
        self.pep_file = 'pep'
        for k, v in bez_conf:
            setattr(self, str(k), v)
        for k, v in options:
            setattr(self, str(k), v)
            print(str(k), ' = ', v)

    def auto_file(self):
        pairs = []
        p = pd.read_csv(self.pairs_file, sep='\n', header=None, nrows=30)
        p = '\n'.join(p[0])
        if 'path length' in p or 'MAXIMUM GAP' in p:
            collinearity = bez.read_colinearscan(self.pairs_file)
            pairs = [[v[0], v[2]] for k in collinearity for v in k[1]]
        elif 'MATCH_SIZE' in p or '## Alignment' in p:
            collinearity = bez.read_mcscanx(self.pairs_file)
            pairs = [[v[0], v[2]] for k in collinearity for v in k[1]]
        elif '# Alignment' in p:
            collinearity = bez.read_coliearity(self.pairs_file)
            pairs = [[v[0], v[2]] for k in collinearity for v in k[1]]
        elif '###' in p:
            collinearity = bez.read_jcvi(self.pairs_file)
            pairs = [[v[0], v[2]] for k in collinearity for v in k[1]]
        elif ',' in p:
            collinearity = pd.read_csv(self.pairs_file, header=None)
            pairs = collinearity.values.tolist()
        else:
            collinearity = pd.read_csv(self.pairs_file, header=None, sep='\t')
            pairs = collinearity.values.tolist()
        df = pd.DataFrame(pairs)
        df = df.drop_duplicates()
        df[0] = df[0].astype(str)
        df[1] = df[1].astype(str)
        df.index = df[0]+','+df[1]
        return df

    def run(self):
        print(u"parent process ID = %d"%os.getpid())
        cpu = cpu_count()
        print("CPU supports %d threads" % (cpu))
        if cpu > 12:
            jc = cpu-5
        else:
            jc = cpu
        pool = multiprocessing.Pool(processes = int(jc/2)) # Step II : 进程池
        if os.path.exists('./processes'):
            pass
        else:
            os.makedirs('./processes')
        df_pairs = self.auto_file()
        # print(df_pairs[-5:])
        m = df_pairs.iloc[:,0].size
        if (m/50.0)-(m/50) == 0:
            n0 = int(m/50)
        else:
            n0 = int((m/50)+1)
        for i in range(n0):
            # print(df_pairs[i:i+500])
            if i < n0-1:
                new_pd = df_pairs[i:i+50]
            else:
                new_pd = df_pairs[i:]
            pool.apply_async(self.run0,(new_pd,i,), error_callback=self.print_error)  # Step III : 异步（并行）计算
        pool.close() # Step IV : 准备结束
        pool.join() # Step IV : 完全结束
        # os.rmdir('processes')

    # 多进程错误打印
    def print_error(self,value):
        print("Process pool error, the cause of the error is :", value)

    def run0(self,df_pairs,n):
        print('The No.',n,' begin the process, process number ',os.getpid())
        prefix = './processes/'+str(n)
        cds = SeqIO.to_dict(SeqIO.parse(self.cds_file, "fasta"))
        pep = SeqIO.to_dict(SeqIO.parse(self.pep_file, "fasta"))
        path = os.getcwd()
        if not os.path.exists(self.pep_file):
            bez.cds_to_pep(os.path.join(path, self.cds_file),
                            os.path.join(path, self.pep_file))
        if os.path.exists(self.ks_file):
            ks_file = open(self.ks_file, 'a+')
        else:
            ks_file = open(self.ks_file, 'w')
            ks_file.write(
                '\t'.join(['id1', 'id2', 'ka_NG86', 'ks_NG86', 'ka_YN00', 'ks_YN00'])+'\n')

        df_pairs = df_pairs[(df_pairs[0].isin(cds.keys())) & (df_pairs[1].isin(
            cds.keys())) & (df_pairs[0].isin(pep.keys())) & (df_pairs[1].isin(pep.keys()))]
        pairs = df_pairs[[0, 1]].to_numpy()
        if len(pairs) > 0 and pairs[0][0][:3] == pairs[0][1][:3]:
            allpairs = []
            pair_hash = {}
            for k in pairs:
                if k[0]+','+k[1] in pair_hash or k[1]+','+k[0] in pair_hash:
                    continue
                else:
                    pair_hash[k[0]+','+k[1]] = 1
                    pair_hash[k[1]+','+k[0]] = 1
                    allpairs.append(k)
            pairs = allpairs
        for k in pairs:
            SeqIO.write([cds[k[0]], cds[k[1]]], prefix+self.pair_cds_file, "fasta")
            SeqIO.write([pep[k[0]], pep[k[1]]], prefix+self.pair_pep_file, "fasta")
            self.read_length(prefix)
            kaks = self.pair_kaks(k,prefix)
            # print(kaks)#####################################
            if kaks == []:
                for file in (prefix+self.pair_pep_file, prefix+self.pair_cds_file, prefix+self.mrtrans, prefix+self.pair_yn, prefix+self.prot_align_file, prefix+'2YN.dN', prefix+'2YN.dS', prefix+'2YN.t', prefix+'rst', prefix+'rst1', prefix+'yn00.ctl', prefix+'rub'):
                    try:
                        os.remove(file)
                        print('remove ',file)
                    except OSError:
                        pass
                continue
            ks_file = open(self.ks_file, 'a+')
            ks_file.write('\t'.join([str(i) for i in list(k)+list(kaks)])+'\n')
            ks_file.close()
        for file in (prefix+self.pair_pep_file, prefix+self.pair_cds_file, prefix+self.mrtrans, prefix+self.pair_yn, prefix+self.prot_align_file, prefix+'2YN.dN', prefix+'2YN.dS', prefix+'2YN.t', prefix+'rst', prefix+'rst1', prefix+'yn00.ctl', prefix+'rub'):
            try:
                os.remove(file)
                print('remove ',file)
            except OSError:
                pass
        print('End of the No.',n,' processes, process number ',os.getpid(),'##########################################################')

    def read_length(self,prefix):
        seq = []
        name = []
        s = ''
        for i in open(prefix+self.pair_pep_file,'r'):
            if i[0] == '>':
                if s != '':
                    seq.append(s)
                name.append(i.strip('\n'))
                s = ''
            else:
                s = s + i.strip('\n')
        if s != '':
            seq.append(s)
            s = ''
        if len(seq[0])%3 != 0 or len(seq[1])%3:
            pass
            # print(str(name[0])[1:],str(name[1])[1:],' Not a multiple of three')
        # print(str(name[0])[1:],len(seq[0])/3.0,str(name[1])[1:],len(seq[1])/3.0)

    def pair_kaks(self, k,prefix):
        self.align(prefix)
        pal = self.pal2nal(prefix)
        if not pal:
            return []
        kaks = self.run_yn00(prefix)
        if kaks == None:
            return []
        kaks_new = [kaks[k[0]][k[1]]['NG86']['dN'], kaks[k[0]][k[1]]['NG86']
                    ['dS'], kaks[k[0]][k[1]]['YN00']['dN'], kaks[k[0]][k[1]]['YN00']['dS']]
        return kaks_new

    def align(self,prefix):
        if self.align_software == 'mafft':
            mafft_cline = MafftCommandline(
                cmd=self.mafft_path, input=self.pair_pep_file, auto=True)
            stdout, stderr = mafft_cline()
            align = AlignIO.read(StringIO(stdout), "fasta")
            AlignIO.write(align, self.prot_align_file, "fasta")
        if self.align_software == 'muscle':
            muscle_cline = MuscleCommandline(
                cmd=self.muscle_path, input=prefix+self.pair_pep_file, out=prefix+self.prot_align_file, seqtype="protein", clwstrict=True)
            # try:
            #     stdout, stderr = muscle_cline()
            # except:
            #     print('error')
            stdout, stderr = muscle_cline()

    def pal2nal(self,prefix):
        args = ['perl', self.pal2nal_path, prefix+self.prot_align_file,
                prefix+self.pair_cds_file, '-output paml -nogap', '>'+prefix+self.mrtrans]
        command = ' '.join(args)
        try:
            os.system(command)
        except:
            return False
        return True

    def run_yn00(self,prefix):
        yn = yn00.Yn00()
        yn.alignment = prefix+self.mrtrans
        yn.out_file = prefix+self.pair_yn
        yn.set_options(icode=0, commonf3x4=0, weighting=0, verbose=1)
        try:
            run_result = yn.run(command=self.yn00_path)
        except:
            run_result = None
        return run_result



# -*- encoding: utf-8 -*-
'''
@File        :family_pair.py
@Time        :2021/09/28 11:23:26
@Author        :charles kiko
@Version        :1.0
@Contact        :charles_kiko@163.com
@Desc        :基因家族关系提取
'''


import csv
import sys
from famCircle.bez import *

class family_pair():
    def __init__(self, options):
        self.pairfile_type = 'blast'
        self.class1 = False
        for k, v in options:
            setattr(self, str(k), v)
            print(k, ' = ', v)

    def read_family(self):
        family_list0 = []
        for i in open(self.family_list,'r',encoding='utf-8'):
            if i[0] != '\n' and i[0] != '#':
                line = i.strip('\n').split()
                if line[0] not in family_list0:
                    family_list0.append(line[0])
        return family_list0

    def run(self):
        list0 = self.read_family()
        f = open(self.savefile,'w',encoding='utf-8')
        if self.pairfile_type == 'blast':
            num = 0
            name_list = []
            for i in open(self.gene_pair,'r',encoding='utf-8'):
                if i != '\n' and i[0] != '#':
                    line = i.strip('\n').split()
                    if line[0] == line[1]:
                        continue
                    if line[0] not in name_list:
                        name_list.append(line[0])
                        if line[0] in list0 or line[1] in list0:
                            f.write(i)
                            num = 1
                        else:
                            continue
                    else:
                        if num <= int(self.parameter):
                            if line[0] in list0 or line[1] in list0:
                                f.write(i)
                                num = 1
                            else:
                                continue
                        else:
                            continue
        if self.pairfile_type == 'MCScanX':
            for row in open(self.gene_pair,'r',encoding='utf-8'):
                if (row[0] == '\n'):
                    continue
                elif(row[:12] == '## Alignment'):
                    lt = row.strip('\n').split(' ')
                    block = {}
                    for i in lt:
                        if '=' in str(i):
                            lt0 = i.split('=')
                            block[str(lt0[0])] = str(lt0[1])
                    N = int(block['N'])
                    if N >= int(self.parameter):
                        self.class1 = True
                    else :
                        self.class1 = False
                elif ('#' not in row):
                    if self.class1:
                        lt = row.strip('\n').split()
                        if len(lt) == 5:
                            id1, id2 = str(lt[2]),str(lt[3])
                        elif len(lt) == 4:
                            id1, id2 = str(lt[1]),str(lt[2])
                        elif len(lt) == 6:
                            id1, id2 = str(lt[3]),str(lt[4])
                        else:
                            print(row)
                            print('Parse error!')
                            exit()
                        if id1 in list0 or id2 in list0:
                            f.write(id1+'\t'+id2+'\n')

        elif self.pairfile_type == 'wgdi':
            for row in open(self.gene_pair,'r',encoding='utf-8'):
                if (row[0] == '\n'):
                    continue
                elif (row[0] == '#'):
                    lt = row.strip('\n').split(' ')
                    block = {}
                    for i in lt:
                        if '=' in str(i):
                            lt0 = i.split('=')
                            block[str(lt0[0])] = str(lt0[1])
                    N = int(block['N'])
                    if N >= int(self.parameter):
                        self.class1 = True
                    else :
                        self.class1 = False
                else:
                    if self.class1:
                        lt = row.strip('\n').split()
                        if lt[0] in list0 or lt[2] in list0:
                            f.write(str(lt[0])+'\t'+str(lt[2])+'\n')
        elif self.pairfile_type == 'ColinearScan':
            for row in open(self.gene_pair,'r',encoding='utf-8'):
                if (row[0] == '\n' and row[0] == '+'):
                    continue
                elif (row[:3] == 'the'):
                    lt = row.strip('\n').split()
                    N = int(lt[-1])
                    if N >= int(self.parameter):
                        self.class1 = True
                    else :
                        self.class1 = False
                else:
                    if self.class1:
                        lt = row.strip('\n').split()
                        if lt[0] in list0 or lt[2] in list0:
                            f.write(str(lt[0])+'\t'+str(lt[2])+'\n')
        else:
            print('pairfile_type error: File Format not recognized!')
            exit()



# -*- encoding: utf-8 -*-
'''
@File        :Ks_allocation.py
@Time        :2021/09/28 11:21:18
@Author        :charles kiko
@Version        :1.0
@Contact        :charles_kiko@163.com
@Desc        :全基因的ks
'''


import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
from famCircle.bez import *
import sys
from scipy.stats.kde import gaussian_kde

class Ks_allocation():
    def __init__(self, options):
        self.vertical = "False"
        self.model = "NG86"
        self.bins = "100"
        self.area = "-0.05,3.5"
        for k, v in options:
            setattr(self, str(k), v)
            print(k, ' = ', v)

    def read_family(self):
        family_list = []
        for line in open(self.family_list,'r'):
            if line != '\n' and '#' not in line:
                line = line.strip('\n').split()
                if line[0] in family_list:
                    continue
                family_list.append(line[0])
        return family_list

    def readks(self,list0):
        read_ks = []
        f = open(self.ks, 'r', encoding='utf-8')
        for row in f:
            if row[0] != '#' and row[0] != '\n':
                row = row.strip('\n').split('\t')
                if row[0] != 'id1' and len(row) != 2:
                    if row[0] in list0 or row[1] in list0:
                        print(row)
                        read_ks.append(row)
        kspd = pd.DataFrame(read_ks)
        kspd.rename(columns={0: 'id1', 1: 'id2',
                                2: 'ka_NG86', 3: 'ks_NG86',
                                4: 'ka_YN00', 5: 'ks_YN00'}, inplace=True)
        return kspd

    # 绘制核密度曲线图
    def KdePlot(self,x):
        # 绘制核密度分布直方图
        plt.figure(figsize=(20,10),dpi=1000)
        plt.grid(c='grey',ls='--',linewidth=0.2)
        dist_space = np.linspace(float(self.area.split(',')[0]),float(self.area.split(',')[1]), 500)
        kdemedian = gaussian_kde(x)
        kdemedian.set_bandwidth(bw_method=kdemedian.factor / 3.)
        plt.fill_between(dist_space, y1=0, y2=kdemedian(dist_space), facecolor='#00a3af', alpha=0.5)
        xx = list(dist_space)[list(kdemedian(dist_space)).index(max(list(kdemedian(dist_space))))]
        xx0 = round(xx,4)
        plt.plot(dist_space, kdemedian(dist_space), color='#00a3af',
                 label='KED_' + str(xx0))
        plt.legend()
        plt.title(self.species_list.replace(",", "_") + ' Ks kernel density estimation')# 设置图片标题
        plt.xlabel('ks')# 设置 x 轴标签
        plt.ylabel('density')# 设置 y 轴标签

    def run(self):
        family_list = self.read_family()
        kspdx = self.readks(family_list)
        if self.model == "NG86":
            kspdx["ks_NG86"] = kspdx["ks_NG86"].astype(float)
            kspdx = kspdx[(kspdx["ks_NG86"] > 0) & (kspdx["ks_NG86"] < 99)]
            y = sorted(list(kspdx["ks_NG86"]))
        elif self.model == "YN00":
            kspdx["ks_YN00"] = kspdx["ks_YN00"].astype(float)
            kspdx = kspdx[(kspdx["ks_YN00"] > 0) & (kspdx["ks_YN00"] < 99)]
            y = sorted(list(kspdx["ks_YN00"]))
        self.KdePlot(y)
        plt.savefig(self.savefile)# 存储图片
        sys.exit(0)



# -*- encoding: utf-8 -*-
'''
@File        :filterWGD.py
@Time        :2021/09/28 11:21:18
@Author        :charles kiko
@Version        :1.0
@Contact        :charles_kiko@163.com
@Desc        :全基因的ks
'''


import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
from famCircle.bez import *
import sys
from scipy.stats.kde import gaussian_kde
from collections import OrderedDict


class filterWGD():
    def __init__(self, options):
        self.area1 = "0,1000"
        self.area2 = "0,1000"
        self.area3 = "0,1000"
        self.area4 = "0,1000"
        self.area5 = "0,1000"
        self.area6 = "0,1000"
        self.area7 = "0,1000"
        self.area8 = "0,1000"
        self.area9 = "0,1000"
        self.area10 = "0,1000"

        for k, v in options:
            setattr(self, str(k), v)
            print(k, ' = ', v)

    def read_colinearity(self):
        colinearity = {}
        list0 = []
        if self.colinearity_type == 'WGDI':
            num = 0
            colinearity_list = []
            for line in open(self.colinearity_file, 'r', encoding='utf-8'):
                if line[:11] == '# Alignment':
                    if len(colinearity_list) == 0:
                        pass
                    else:
                        dic_0 = {}
                        dic_0['length'] = length
                        dic_0['name'] = num
                        dic_0['pvalue'] = pvalue
                        dic_0['list'] = colinearity_list
                        colinearity[num] = dic_0
                        list0.append(num)
                        colinearity_list = []
                    num += 1
                    name = num
                    lt = line.strip('\n').split()
                    dic_ = {}
                    for i in lt:
                        if '=' in i:
                            lt0 = i.split('=')
                            dic_[str(lt0[0])] = lt0[1]
                    length = int(dic_['N'])
                    pvalue = float(dic_['pvalue'])
                else:
                    lt = line.strip('\n').split()
                    colinearity_list.append([lt[0],lt[2]])

        elif self.colinearity_type == 'MCScanX':
            num = 0
            colinearity_list = []
            for line in open(self.colinearity_file, 'r', encoding='utf-8'):
                if line[:13] == '## Alignment ':
                    # print(colinearity_list)
                    if len(colinearity_list) == 0:
                        pass
                    else:
                        dic_0 = {}
                        dic_0['length'] = length
                        dic_0['name'] = num
                        dic_0['pvalue'] = pvalue
                        dic_0['list'] = colinearity_list
                        colinearity[num] = dic_0
                        list0.append(num)
                        colinearity_list = []
                    num += 1
                    name = num
                    lt = line.strip('\n').split()
                    dic_ = {}
                    for i in lt:
                        if '=' in i:
                            lt0 = i.split('=')
                            dic_[str(lt0[0])] = lt0[1]
                    length = int(dic_['N'])
                    pvalue = float(dic_['e_value'])
                elif line[0] == '#' or line[0] == '\n':
                    continue
                else:
                    lt = line.strip('\n').split()
                    if len(lt) == 4:
                        colinearity_list.append([lt[1],lt[2]])
                    elif len(lt) == 5:
                        colinearity_list.append([lt[2],lt[3]])
                    elif len(lt) == 6:
                        colinearity_list.append([lt[3],lt[4]])
        else:
            print('blockfile_type error: File Format not recognized!')
            exit()

        if len(colinearity_list) == 0:
            pass
        else:
            dic_0 = {}
            dic_0['length'] = length
            dic_0['name'] = num
            dic_0['pvalue'] = pvalue
            dic_0['list'] = colinearity_list
            colinearity[num] = dic_0
            list0.append(num)
            colinearity_list = []


        dicx = {}
        listx = []
        for i in list0:
            dicx[i] = int(colinearity[i]['length'])
        dic1SortList = sorted(dicx.items(),key = lambda x:x[1],reverse = True)# 降序
        for i in dic1SortList:
            listx.append(i[0])
        return colinearity,listx

    def colinearity_(self,colinearity,gff,listx):# 共线性拆分
        colinearity0 = {}
        n = 0
        for i in colinearity.keys():
            lt = []
            for j in colinearity[i]['list']:
                id1,id2 = j[0],j[1]
                if gff[id1]['chr'] != gff[id2]['chr']:
                    n += 1
                    colinearity0[n] = colinearity[i]
                    break
                else:
                    if abs(gff[id1]['order']-gff[id2]['order'])<15:
                        if len(lt) == 0:
                            continue
                        else:
                            n += 1
                            colinearity0[n]=colinearity[i]
                            colinearity0[n]['length'] = len(lt)
                            colinearity0[n]['list'] = lt
                            lt = []
                    else:
                        lt.append(j)
        dicx = {}
        list0 = []
        for i in colinearity0.keys():
            dicx[i] = int(colinearity0[i]['length'])
        dic1SortList = sorted(dicx.items(),key = lambda x:x[1],reverse = True)# 降序
        for i in dic1SortList:
            # print(int(colinearity0[i[0]]['length']))
            list0.append(i[0])
        return colinearity0,list0

    def read_min_ks(self):
        if self.min_ks == '.0':# 不包括0
            op = 1
        elif self.min_ks == '0.':# 包括0
            op = 2
        else:
            op = 3


    def readks(self):
        ks = {}
        op = self.read_min_ks()
        f = open(self.ks, 'r', encoding='utf-8')
        for row in f:
            if row[0] != '#' and row[0] != '\n':
                row = row.strip('\n').split('\t')
                if row[0] != 'id1' and len(row) != 2:
                    if op == 1: 
                        if float(row[3]) > 0.0:
                            name = str(row[0]) + '_' + str(row[1])
                            ks[name] = row[3]
                        else:
                            continue
                    elif op == 2:
                        if float(row[3]) >= 0.0:
                            name = str(row[0]) + '_' + str(row[1])
                            ks[name] = row[3]
                        else:
                            continue
                    else:
                        if float(row[3]) > float(self.min_ks):
                            name = str(row[0]) + '_' + str(row[1])
                            ks[name] = row[3]
                        else:
                            continue
        return ks

    def area_(self,ks0):
        area0 = [self.area1,self.area2,self.area3,self.area4,self.area5,self.area6,self.area7,self.area8,self.area9,self.area10]
        for i in range(int(self.multiple)):
            lt = area0[i].split(',')
            if ks0 > float(lt[0]) and ks0 < float(lt[1]):
                # print(float(lt[0]),float(lt[1]))
                return True
        return False
    
    def block_ks(self,ks,lit):
        num = 0
        sum0 = 0.0
        for i in lit:
            if str(i[0]) + '_' + str(i[1]) in ks.keys():
                num += 1
                sum0 = sum0 + float(ks[str(i[0]) + '_' + str(i[1])])
            elif str(i[1]) + '_' + str(i[0]) in ks.keys():
                num += 1
                sum0 = sum0 + float(ks[str(i[1]) + '_' + str(i[0])])
            else:
                continue
        # print(sum0,num)
        if sum0 == 0:
            return 0
        avg = float(sum0/float(num))
        return avg

    def found_block(self,list0,colinearity,ks):
        out_block = []
        for i in range(len(list0)):
            if i+2 <= len(list0):
                num = 0
                sum0 = 0
                # print(colinearity[list0[i]]['list'])
                ltxx = colinearity[list0[i]]['list']+colinearity[list0[i+1]]['list']+colinearity[list0[i+2]]['list']
                for lt0 in ltxx:
                    if str(lt0[0]) + '_' + str(lt0[1]) in ks.keys():
                        ks0 = ks[str(lt0[0]) + '_' + str(lt0[1])]
                        num += 1
                        sum0 = sum0 + float(ks0)
                    elif str(lt0[1]) + '_' + str(lt0[0]) in ks.keys():
                        ks0 = ks[str(lt0[1]) + '_' + str(lt0[0])]
                        num += 1
                        sum0 = sum0 + float(ks0)
                    else:
                        pass
                if sum0 == 0:
                    continue
                ks_a = sum0/(num*1.0)
                if not self.area_(ks_a):
                    min_block_ = colinearity[list0[i+2]]['length']
                    # out_block.append(colinearity[list0[i+2]]['name'])
                    if int(min_block_) > int(self.block):
                        out_block.append(colinearity[list0[i+2]]['name'])
                        continue
                    return min_block_,out_block
            else:
                min_block_ = colinearity[list0[-1]]['length']
                return min_block_,out_block

    def writefile(self,colinearity,list0,min_block_,out_block,ks):
        # print(min_block_)
        f = open(self.savefile, 'w', encoding='utf-8')
        num = 0
        for i in range(len(list0)):
            # print(colinearity[list0[i]]['length'],list0[i])
            if int(colinearity[list0[i]]['length']) > int(min_block_) and colinearity[list0[i]]['name'] not in out_block:# and colinearity[list0[i]]['name'] not in out_block
                num += 1
                # print(num,colinearity[list0[i]]['length'],list0[i],self.block_ks(ks,colinearity[list0[i]]['list']))
                
                f.write('# Alignment '+str(num) +': pvalue='+str(colinearity[list0[i]]['pvalue'])+' N='+str(colinearity[list0[i]]['length'])+'\n')
                for n in colinearity[list0[i]]['list']:
                    f.write(str(n[0]) + '\t' + str(n[1])+'\n')
            # Alignment 1: score=259464.0 pvalue=0.0237 N=5774

    def run(self):
        gff = read_gff(self.gff)
        colinearity,list0 = self.read_colinearity()
        colinearity,list0 = self.colinearity_(colinearity,gff,list0)
        # print(colinearity)
        ks = self.readks()
        min_block_,out_block = self.found_block(list0,colinearity,ks)
        # print(out_block)
        self.writefile(colinearity,list0,min_block_,out_block,ks)
        sys.exit(0)



# -*- encoding: utf-8 -*-
'''
@File        :circle_family.py
@Time        :2021/09/28 11:25:41
@Author        :charles kiko
@Version        :1.0
@Contact        :charles_kiko@163.com
@Desc        :家族分布圈图
'''


import re
import sys
from math import *
import gc
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
from matplotlib.patches import *
from matplotlib.patches import Circle, Ellipse
from pylab import *
from collections import Counter
from famCircle.bez import *


class circle_family():
    def __init__(self, options):
    ##### circle parameters
        self.GAP_RATIO = 2 #gaps between chromosome circle, chr:gap = 4: 1
        self.radius = 0.33
        # self.block=0.01 #block scize
        # self.blockthick = 0.004 #0.006
        self.shiftratio = -2.1 # define the distance between overlapping glocks
        self.specieslist = []
        self.iscompletegenome = {}
        self.gene2pos={}
        self.gene2chain = {}
        self.chro2len = {}
        self.otherchrolist = []
        self.genepairsfile_type = 'WGDI'
        self.labels = []
        self.genes = []
        self.genepair2Ks = {}
        self.genepair2Ka = {}
        self.block = '0'
        self.class1 = True
        self.start_list = []
        self.colornum = 1
        self.class1 = True
        # self.color = [ '#a0d8ef', '#c7b370', '#eb6ea5', '#0094c8', '#b8d200', '#00a497'
        #               , '#d9333f', '#3e62ad', '#aacf53', '#f6ad49', '#e9e4d4', '#f39800'
                      # , '#98d98e', '#ffd900', '#68be8d', '#745399', '#028760', '#c0a2c7'],"#eaedf7"
        self.color = ["#2ca9e1","#ffd900","#e2041b","#38b48b","#ea5506","#8f2e14"
                      ,"#b44c97","#f5b1aa","#bce2e8","#82ae46","#824880","#5a544b","#4d4c61"
                      ,"#6f4b3e","#192f60","#2b2b2b","#475950","#727171","#2c4f54","#455765","#432f2f"]
        for k, v in options:
            setattr(self, str(k), v)
            print(k, ' = ', v)

    def ksrun(self):
        fpchrolen1 = open(self.lens1,'r', encoding='utf-8')
        fpchrolen2 = open(self.lens2,'r', encoding='utf-8')
        fpgff1 = open(self.gff1,'r', encoding='utf-8')
        fpgff2 = open(self.gff2,'r', encoding='utf-8')
        #### gene block parameters
        figure(1, (8, 8))  ### define the a square, or other rectangle of the figure, if to produce an oval here
        root =axes([0, 0, 1, 1])
        if self.species1 == self.species2:
            lengthset = set()
            for i in [self.species1,self.species2]:
                lengthset.add(len(i))
            # print(lengthset)# 物种名字长度
            chrolist = []
            for row in fpchrolen1:
                chro = row.split('\t')[0]
                for i in lengthset:
                    if (chro[:i] in [self.species1,self.species2]):
                        chrolist.append(chro)
                    else:
                        pass
            fpchrolen1.close()
            # print(chrolist)
            for i in range(len(chrolist)):
                string = chrolist[i]
            #   print string[0:2]
                isnew = 1
                for sp in self.specieslist:
                    if sp == string[0:2]:
                        isnew = 0
                        break
                if isnew==1:
                    self.specieslist.append(string[0:2])
                    if string == string[0:2]:
                        self.iscompletegenome[string[0:2]] = 1
                    else:
                        self.iscompletegenome[string[0:2]] = 0
            # print(self.specieslist)
            # print(self.iscompletegenome)
            ### input chromosome length
            fpchrolen1 = open(self.lens1,'r', encoding='utf-8')
            for row in fpchrolen1:
                if row[0] == '#' or row == '\n':
                    continue
                chro,length = row.split('\t')[0],row.split('\t')[1]
                if len(chro) > 10 :
                    continue
                sp = chro[:2]
                if self.iscompletegenome[sp] == 1 :
                    self.chro2len[chro] = int(length)
                    self.otherchrolist.append(chro)
                else:
                    if chro in chrolist :
                        self.chro2len[chro] = int(length)
                        self.otherchrolist.append(chro)
            fpchrolen1.close()
            # print(self.chro2len,'self.chro2len')
            # print(self.otherchrolist,'self.otherchrolist')
            ### full chro list
            for i in self.otherchrolist:
                self.labels.append(i)
            # print('self.labels',self.labels)
            for row in fpgff1:
                ch, gene, start, end = row.split()[0],row.split()[1],row.split()[2],row.split()[3]
                gene = readname(gene,self.species1+self.species2)
                start = int(start)
                end = int(end)
                self.gene2pos[gene] = int(start)
                # print(start)
                self.gene2chain[gene] = int((end-start)/abs(end -start))
            fpgff1.close()

        elif self.species1 != self.species2:
            lengthset = set()
            for i in [self.species1,self.species2]:
                lengthset.add(len(i))
            # print(lengthset)# 物种名字长度
            chrolist = []
            for row in fpchrolen1:
                chro = row.split('\t')[0]
                for i in lengthset:
                    if (chro[:i] in [self.species1,self.species2]):
                        chrolist.append(chro)
                    else:
                        pass
            fpchrolen1.close()
            for row in fpchrolen2:
                chro = row.split('\t')[0]
                for i in lengthset:
                    if (chro[:i] in [self.species1,self.species2]):
                        chrolist.append(chro)
                    else:
                        pass
            fpchrolen2.close()
            # print(chrolist)
            for i in range(len(chrolist)):
                string = chrolist[i]
            #   print string[0:2]
                isnew = 1
                for sp in self.specieslist:
                    if sp == string[0:2]:
                        isnew = 0
                        break
                if isnew==1:
                    self.specieslist.append(string[0:2])
                    if string == string[0:2]:
                        self.iscompletegenome[string[0:2]] = 1
                    else:
                        self.iscompletegenome[string[0:2]] = 0
            # print(self.specieslist)
            # print(self.iscompletegenome)
            ### input chromosome length
            fpchrolen1 = open(self.lens1,'r', encoding='utf-8')
            for row in fpchrolen1:
                if row[0] == '#' or row == '\n':
                    continue
                chro,length = row.split('\t')[0],row.split('\t')[1]
                if len(chro) > 10 :
                    continue
                sp = chro[:2]
                if self.iscompletegenome[sp] == 1 :
                    self.chro2len[chro] = int(length)
                    self.otherchrolist.append(chro)
                else:
                    if chro in chrolist :
                        self.chro2len[chro] = int(length)
                        self.otherchrolist.append(chro)
            fpchrolen1.close()
            fpchrolen2 = open(self.lens2,'r', encoding='utf-8')
            for row in fpchrolen2:
                if row[0] == '#' or row == '\n':
                    continue
                chro,length = row.split('\t')[0],row.split('\t')[1]
                if len(chro) > 10 :
                    continue
                sp = chro[:2]
                if self.iscompletegenome[sp] == 1 :
                    self.chro2len[chro] = int(length)
                    self.otherchrolist.append(chro)
                else:
                    if chro in chrolist :
                        self.chro2len[chro] = int(length)
                        self.otherchrolist.append(chro)
            fpchrolen2.close()
            # print(self.chro2len,'self.chro2len')
            # print(self.otherchrolist,'self.otherchrolist')
            ### full chro list
            for i in self.otherchrolist:
                self.labels.append(i)
            # print('self.labels',self.labels)

            for row in fpgff1:
                ch, gene, start, end = row.split()[0],row.split()[1],row.split()[2],row.split()[3]
                gene = readname(gene,self.species1+self.species2)
                start = int(start)
                end = int(end)
                self.gene2pos[gene] = int(start)
                # print(start)
                self.gene2chain[gene] = int((end-start)/abs(end -start))
            fpgff1.close()

            for row in fpgff2:
                ch, gene, start, end = row.split()[0],row.split()[1],row.split()[2],row.split()[3]
                gene = readname(gene,self.species1+self.species2)
                start = int(start)
                end = int(end)
                self.gene2pos[gene] = int(start)
                # print(start)
                self.gene2chain[gene] = int((end-start)/abs(end -start))
            fpgff2.close()
        return root

    def rad_to_coord(self, angle, radius):
        return radius*cos(angle), radius*sin(angle)

    def to_radian(self, bp, total):
        # from basepair return as radian
        return radians(bp*360./total)

    def plot_arc(self, start, stop, radius):
        # start, stop measured in radian
        t = arange(start, stop, pi/720.)
        x, y = radius*cos(t), radius*sin(t)
        plot(x, y, '-', color='#ffea00', alpha=.5)# 染色体圆弧

    def plot_cap(self, angle, clockwise):
        radius=self.sm_radius
        # angle measured in radian, clockwise is boolean
        if clockwise: 
            t = arange(angle, angle+pi, pi/30.)
        else: 
            t = arange(angle, angle-pi, -pi/30.)
        x, y = radius*cos(t), radius*sin(t)
        middle_r = (self.radius_a+self.radius_b)/2
        x, y = x + middle_r*cos(angle), y + middle_r*sin(angle)
        plot(x, y, '-', color='#ffea00', alpha=.5)# 边缘

    def zj(self):
        fullchrolen = int(pd.DataFrame(self.chro2len.values()).sum())
        chr_number = len(self.labels) # total number of chromosomes
        GAP = fullchrolen/self.GAP_RATIO/chr_number # gap size in base pair
        total_size = fullchrolen + chr_number * GAP # base pairs
        for i in range(chr_number):
            self.start_list.append(0)
        for i in range(1, chr_number):
            self.start_list[i] = self.start_list[i-1] + self.chro2len[self.labels[i-1]] + GAP
        stop_list = [(self.start_list[i] + self.chro2len[self.labels[i]]) for i in range(chr_number)]
        return stop_list, total_size, chr_number

    def transform_pt(self, ch, pos, r, total_size):
        rad = self.to_radian(pos + self.start_list[ch], total_size)
        return r*cos(rad), r*sin(rad)

    def transform_pt0(self, ch, pos, r, total_size):
        rad = self.to_radian(pos + self.start_list[ch], total_size)
        return r*cos(rad), r*sin(rad),rad

    def readblast(self):
        one_gene = []
        alphagenepairs = open(self.genepairs, 'r', encoding='utf-8')
        chrnum = []
        if self.genepairsfile_type == 'famCircle':
            for row in alphagenepairs:
                if (row[0] == '\n'):
                    continue
                elif (row[0] == '#'):
                    lt = row.strip('\n').split(' ')
                    block = {}
                    for i in lt:
                        if '=' in str(i):
                            lt0 = i.split('=')
                            block[str(lt0[0])] = str(lt0[1])
                    N = int(block['N'])
                    if N >= int(self.block):
                        self.class1 = True
                    else :
                        self.class1 = False
                else:
                    if self.class1:
                        lt = row.strip('\n').split()
                        id1, id2 = str(lt[0]),str(lt[1])
                        id1 = readname(id1,self.species1+self.species2)
                        id2 = readname(id2,self.species1+self.species2)
                        if (id1 not in self.gene2pos.keys() or id2 not in self.gene2pos.keys()):
                            continue
                        chro1 = id1.split("^")[0]
                        chro2 = id2.split("^")[0]
                        if (chro1 not in self.labels or chro2 not in self.labels):
                            continue
                        one_gene.append([id1,id2])
                        chr0 = str(id1.split('^')[0])
                        if chr0 not in chrnum:
                            chrnum.append(chr0)
                        else:
                            pass
        elif self.genepairsfile_type == 'WGDI':
            for row in alphagenepairs:
                if (row[0] == '\n'):
                    continue
                elif (row[0] == '#'):
                    lt = row.strip('\n').split(' ')
                    block = {}
                    for i in lt:
                        if '=' in str(i):
                            lt0 = i.split('=')
                            block[str(lt0[0])] = str(lt0[1])
                    N = int(block['N'])
                    if N >= int(self.block):
                        self.class1 = True
                    else :
                        self.class1 = False
                else:
                    if self.class1:
                        lt = row.strip('\n').split()
                        id1, id2 = str(lt[0]),str(lt[2])
                        id1 = readname(id1,self.species1+self.species2)
                        id2 = readname(id2,self.species1+self.species2)
                        if (id1 not in self.gene2pos.keys() or id2 not in self.gene2pos.keys()):
                            continue
                        chro1 = id1.split("^")[0]
                        chro2 = id2.split("^")[0]
                        if (chro1 not in self.labels or chro2 not in self.labels):
                            continue
                        one_gene.append([id1,id2])
                        chr0 = str(id1.split('^')[0])
                        if chr0 not in chrnum:
                            chrnum.append(chr0)
                        else:
                            pass
        elif self.genepairsfile_type == 'ColinearScan':
            for row in alphagenepairs:
                if (row[0] == '\n' and row[0] == '+'):
                    continue
                elif (row[:3] == 'the'):
                    lt = row.strip('\n').split()
                    N = int(lt[-1])
                    # block = {}
                    # for i in lt:
                    #     if '=' in str(i):
                    #         lt0 = i.split('=')
                    #         block[str(lt0[0])] = str(lt0[1])
                    # N = int(block['N'])
                    if N >= int(self.block):
                        self.class1 = True
                    else :
                        self.class1 = False
                else:
                    if self.class1:
                        lt = row.strip('\n').split()
                        id1, id2 = str(lt[0]),str(lt[2])
                        id1 = readname(id1,self.species1+self.species2)
                        id2 = readname(id2,self.species1+self.species2)
                        if (id1 not in self.gene2pos.keys() or id2 not in self.gene2pos.keys()):
                            continue
                        chro1 = id1.split("^")[0]
                        chro2 = id2.split("^")[0]
                        if (chro1 not in self.labels or chro2 not in self.labels):
                            continue
                        one_gene.append([id1,id2])
                        chr0 = str(id1.split('^')[0])
                        if chr0 not in chrnum:
                            chrnum.append(chr0)
                        else:
                            pass
        elif self.genepairsfile_type == 'MCScanX':
            for row in alphagenepairs:
                if (row[0] == '\n'):
                    continue
                elif(row[:12] == '## Alignment'):
                    lt = row.strip('\n').split(' ')
                    block = {}
                    for i in lt:
                        if '=' in str(i):
                            lt0 = i.split('=')
                            block[str(lt0[0])] = str(lt0[1])
                    N = int(block['N'])
                    if N >= int(self.block):
                        self.class1 = True
                    else :
                        self.class1 = False
                elif ('#' not in row):
                    if self.class1:
                        lt = row.strip('\n').split()
                        # print(lt)
                        id1, id2 = str(lt[2]),str(lt[3])
                        id1 = readname(id1,self.species1+self.species2)
                        id2 = readname(id2,self.species1+self.species2)
                        if (id1 not in self.gene2pos.keys() or id2 not in self.gene2pos.keys()):
                            continue
                        chro1 = id1.split("^")[0]
                        chro2 = id2.split("^")[0]
                        if (chro1 not in self.labels or chro2 not in self.labels):
                            continue
                        one_gene.append([id1,id2])
                        chr0 = str(id1.split('^')[0])
                        if chr0 not in chrnum:
                            chrnum.append(chr0)
                        else:
                            pass
                    else:
                        continue
        elif self.genepairsfile_type == 'blast':
            num = 0
            name0_list = []
            for row in alphagenepairs:
                if (row[0] == '\n'):
                    continue
                elif ('#' not in row):
                    lt = row.strip('\n').split()
                    # print(lt)
                    if str(lt[0]) not in name0_list:
                        name0_list.append(str(lt[0]))
                        num = 1
                    else:
                        if num <= int(self.block):
                            pass
                        else:
                            continue
                    id1, id2 = str(lt[0]),str(lt[1])
                    id1 = readname(id1,self.species1+self.species2)
                    id2 = readname(id2,self.species1+self.species2)
                    if (id1 not in self.gene2pos.keys() or id2 not in self.gene2pos.keys()):
                        continue
                    chro1 = id1.split("^")[0]
                    chro2 = id2.split("^")[0]
                    if (chro1 not in self.labels or chro2 not in self.labels):
                        continue
                    one_gene.append([id1,id2])
                    chr0 = str(id1.split('^')[0])
                    if chr0 not in chrnum:
                        chrnum.append(chr0)
                    else:
                        pass
        else:
            print('genepairsfile_type error: File Format not recognized!')
            exit()
        self.colornum = len(chrnum)
        alphagenepairs.close()
        blastlist = sorted(one_gene,key=(lambda x:x[0]))
        return blastlist

    def plot_bez_inner(self, p1, p2, cl, total_size,alp):
    #    print "inner"
        a, b, c = p1
        ex1x, ex1y = self.transform_pt(a, b, c, total_size)
        a, b, c = p2
        ex2x, ex2y = self.transform_pt(a, b, c, total_size)
        # Bezier ratio, controls curve, lower ratio => closer to center
        ratio = .5
        x = [ex1x, ex1x*ratio, ex2x*ratio, ex2x]
        y = [ex1y, ex1y*ratio, ex2y*ratio, ex2y]
        step = .01
        t = arange(0, 1+step, step)
        xt = Bezier(x, t)
        yt = Bezier(y, t)
        plot(xt, yt, '-', color=cl, lw=1.6, alpha = alp)#alpha 

    def run(self):
        self.radius_a = float(self.radius)
        self.radius_b = self.radius_a + 0.005
        self.sm_radius=(self.radius_b-self.radius_a)/2 #telomere capping
        root = self.ksrun()
        stop_list, total_size, chr_number = self.zj()
        ## sort gene according to lacation on circle
        blastlist = self.readblast()
        rowno = 0
        chrlist = []
        # for i in range(3):
        list000 = []
        list00 = []
        family_list = open(self.family_list, 'r', encoding='utf-8')
        for row in family_list:
            if row != '\n' and row[0] != '#':
                list00.append(readname(str(row.strip('\n').split()[0]),self.species1+self.species2))
        # ['lja1^04212','lja2^00910','lja2^04322','lja2^04914','lja4^01025','lja4^02196','lja5^00060','lja7^01239'
        #   ,'lja8^00488','lja8^01183','lja9^02201','lja9^02588']# utg857g00011,utg857g00009,utg857g00008,utg124g00067
        for lt in blastlist:
            id1, id2 = str(lt[0]),str(lt[1])
            pos1 = self.gene2pos[id1]
            pos2 = self.gene2pos[id2]
            chro1 = id1.split("^")[0]
            chro2 = id2.split("^")[0]
            sp1 = chro1[0:2]
            sp2 = chro2[0:2]
            if chro1 in chrlist:
                col = self.color[chrlist.index(chro1)]
            else:
                chrlist.append(chro1)
                col = self.color[len(chrlist) - 1]
            order1 = self.labels.index(chro1)
            order2 = self.labels.index(chro2)
            alp = 0.05
            if id1 in list00 or id2 in list00:
                list000.append([id1, id2])
            else:
                self.plot_bez_inner((order1, pos1, self.radius_a*0.85), (order2, pos2, self.radius_a*0.85), '#e9dfe5', total_size,alp)
                #e9dfe5
        ltx = []
        for lt in list000:
            id1, id2 = str(lt[0]),str(lt[1])
            pos1 = self.gene2pos[id1]
            pos2 = self.gene2pos[id2]
            chro1 = id1.split("^")[0]
            chro2 = id2.split("^")[0]
            sp1 = chro1[0:2]
            sp2 = chro2[0:2]
            if chro1 in chrlist:
                col = self.color[chrlist.index(chro1)]
            else:
                chrlist.append(chro1)
                col = self.color[len(chrlist) - 1]
            order1 = self.labels.index(chro1)
            order2 = self.labels.index(chro2)
            alp = 0.05
            self.plot_bez_inner((order1, pos1, self.radius_a*0.85), (order2, pos2, self.radius_a*0.85), "#e60033", total_size,1)
            a, b, c = order1, pos1, self.radius_a*0.92
            ex1x, ex1y,rad1 = self.transform_pt0(a, b, c, total_size)
            a, b, c = order2, pos2, self.radius_a*0.92
            ex2x, ex2y,rad2 = self.transform_pt0(a, b, c, total_size)
            # print(rad1,rad2)
            if id1 not in ltx:
                ltx.append(id1)
                text(ex1x, ex1y, str(id1), horizontalalignment="center", verticalalignment="center", fontsize = 2, color = 'black',rotation=(rad1/(2*pi))*360)
            if id2 not in ltx:
                ltx.append(id2)
                text(ex2x, ex2y, str(id2), horizontalalignment="center", verticalalignment="center", fontsize = 2, color = 'black',rotation=(rad2/(2*pi))*360)
                #e9dfe5

            rowno = rowno + 1
        # the chromosome layout
        j = 0
        for start, stop in zip(self.start_list, stop_list):
            start, stop = self.to_radian(start, total_size), self.to_radian(stop, total_size)
            # shaft
            self.plot_arc(start, stop, self.radius_a)
            self.plot_arc(start, stop, self.radius_b)
            # telemere capping
            clockwise=False
            self.plot_cap(start, clockwise)
            clockwise=True
            self.plot_cap(stop, clockwise)
            # chromosome self.labels
            label_x, label_y = self.rad_to_coord((start+stop)/2, self.radius_b*1.1)# text
            #print label_x, label_y
            text(label_x, label_y, self.labels[j], horizontalalignment="center", verticalalignment="center", fontsize = 7, color = 'black')
            j+=1
        ########
        root.set_xlim(-.8, .8)#-.5, .5
        root.set_ylim(-.8, .8)
        root.set_axis_off()
        savefig(self.savefile, dpi=1000)
        sys.exit(0)



# -*- encoding: utf-8 -*-
'''
@File        :outer.py
@Time        :2021/09/28 11:17:18
@Author        :charles kiko
@Version        :1.0
@Contact        :charles_kiko@163.com
@Desc        :放射型圈图
'''


import re
import os
import sys
import gc
from math import *
import csv
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
from matplotlib.patches import *
from matplotlib.patches import Circle, Ellipse
from pylab import *
from famCircle.bez import *


class outer():
    def __init__(self, options):
    ##### circle parameters
        self.file_type = "coliearity"
        self.GAP_RATIO = 5 #gaps between chromosome circle, chr:gap = 4: 1
        self.radius = 0.33
        self.space = "0.005" # radian 0.01
        self.block= 0.005 #block scize
        self.blockthick = 0.003 #0.006
        self.shiftratio = -2.1 # define the distance between overlapping glocks
        self.specieslist = []
        self.iscompletegenome = {}
        self.gene2pos={}# 基因起始位置
        self.chro2len = {}
        self.otherchrolist = []
        self.labels = []
        self.genes = []
        self.genepair2Ks = {}
        self.orderlist = {}
        self.genepair0 = []
        self.peripheral = 'False'
        self.clusters = None
        self.start_list = []
        for k, v in options:
            setattr(self, str(k), v)
            print(k, ' = ', v)

    def ksrun(self):
        # figurefile = "_".join(sys.argv[1:len(sys.argv)])+".genefam"
        fpchrolen = open(self.lens,'r', encoding='utf-8')
        fpgff = open(self.gff,'r', encoding='utf-8')
        fpgenefamilyinf = open(self.ks, 'r', encoding='utf-8')
        alphagenepairs = open(self.genefamily, 'r', encoding='utf-8')
        #### gene block parameters
        figure(1, (8, 8))  ### define the a square, or other rectangle of the figure, if to produce an oval here
        root =axes([0, 0, 1, 1])
        # chrolist = self.chrolist.split(',')
        chrolist = []
        for row in fpchrolen:
            chro,order_ = row.strip('\n').split('\t')[0],row.split('\t')[2]
            # print(chro)
            chrolist.append(chro)
            self.orderlist[chro] = int(order_)
        fpchrolen.close()
        for i in range(len(chrolist)):
            string = chrolist[i]
        #   print string[0:2]
            isnew = 1
            for sp in self.specieslist:
                if sp == string[0:2]:
                    isnew = 0
                    break
            if isnew==1:
                self.specieslist.append(string[0:2])
                if string == string[0:2]:
                    self.iscompletegenome[string[0:2]] = 1
                else:
                    self.iscompletegenome[string[0:2]] = 0

        ### input chromosome length
        # print(self.iscompletegenome.keys())
        fpchrolen = open(self.lens,'r', encoding='utf-8')
        for row in fpchrolen:
            chro,length = row.split('\t')[0],row.split('\t')[1]# 选择order或者end
            # sp = chro[:2].upper()
            sp = chro[:2]
            if self.iscompletegenome[sp] == 1 :
                self.chro2len[chro] = int(length)
                self.otherchrolist.append(chro)
            else:
                if chro in chrolist :
                    self.chro2len[chro] = int(length)
                    self.otherchrolist.append(chro)
        fpchrolen.close()
        ### full chro list
        for i in self.otherchrolist:
            self.labels.append(i)# 染色体列表
        ## input gff
        for row in fpgff:
            ch, gene, start, end = row.split()[0],row.split()[1],row.split()[2],row.split()[3]
            gene = readname(gene,self.species)
            if gene.split('^')[0] not in self.labels:#筛选染色体
                continue
            start = int(start)
            end = int(end)
            self.gene2pos[gene] = int(start)#基因对应位置
        fpgff.close()
        ### input gene family gene1 gene2 Ka Ks
        for row in fpgenefamilyinf:#ks文件
            if (row.split()[0] == 'id1' or len(row.split()) < 4):
                continue
            gene1, gene2, Ka, Ks = row.split()[0],row.split()[1],row.split()[2],row.split()[3]
            # print(gene1,'********************************')
            # print('1')
            gene1 = readname(gene1,self.species)
            gene2 = readname(gene2,self.species)
            if (gene1 not in self.gene2pos.keys() or gene2 not in self.gene2pos.keys()):
                continue
            genepair = ""
            if gene1 < gene2:
               genepair = gene1 + " " + gene2
            else:
               genepair = gene2 + " " + gene1
            if len(gene1.split("^")[0]) < 10  and len(gene2.split("^")[0]) < 10 :
               # self.genepair2Ka[genepair] = float(Ka)
               self.genepair2Ks[genepair] = float(Ks)#基因对的ks
        fpgenefamilyinf.close()
        # print("genes ", self.genes, "genes")

        for row in alphagenepairs:
            if (row != '\n'):
                familyname = readname(str(row.strip('\n')),self.species)
                if familyname not in self.gene2pos.keys():
                    continue
                if familyname not in self.genes:
                    self.genes.append(familyname)
        return root

    def rad_to_coord(self, angle, radius):
        return radius*cos(angle), radius*sin(angle)

    def to_deg(self, bp, total):
        # from basepair return as degree
        return bp*360./total

    def to_radian(self, bp, total):
        # from basepair return as radian
        # print ("to_radian", bp, total)
        return radians(bp*360./total)

    def plot_arc(self, start, stop, radius):
        # start, stop measured in radian 染色体
        #print start, stop
        t = arange(start, stop, pi/720.)
        x, y = radius*cos(t), radius*sin(t)
        plot(x, y, "k-",alpha=.5)

    def plot_cap(self, angle, clockwise):
        radius=self.sm_radius
        # angle measured in radian, clockwise is boolean 鸭舌
        if clockwise: 
            t = arange(angle, angle+pi, pi/30.)
        else: 
            t = arange(angle, angle-pi, -pi/30.)
        x, y = radius*cos(t), radius*sin(t)
        middle_r = (self.radius_a+self.radius_b)/2
        x, y = x + middle_r*cos(angle), y + middle_r*sin(angle)
        plot(x, y, "k-", alpha=.5)

    def plot_arc_block1(self, start, radius, end):# block
        t = arange(start, start+end, pi/720.)
        x,y = radius * cos(t), radius*sin(t)
        x1, y1 = (radius-self.blockthick) * cos(t), (radius-self.blockthick) * sin(t)
        plot(x, y, "g-", alpha=0.5)

    def plot_arc_block(self, start, radius):# block
        t = arange(start, start+self.block, pi/720.)
        x,y = radius * cos(t), radius*sin(t)
        x1, y1 = (radius-self.blockthick) * cos(t), (radius-self.blockthick) * sin(t)
        plot(x, y, "b-", alpha=0.5)

    def zj(self):
        fullchrolen = int(pd.DataFrame(self.chro2len.values()).sum())
        chr_number = len(self.labels) # total number of chromosomes
        GAP = fullchrolen/self.GAP_RATIO/chr_number # gap size in base pair
        total_size = fullchrolen + chr_number * GAP # base pairs 
        # print('total_size',total_size)
        for i in range(chr_number):
            self.start_list.append(0)
        for i in range(1, chr_number):
            self.start_list[i] = self.start_list[i-1] + self.chro2len[self.labels[i-1]] + GAP
        stop_list = [(self.start_list[i] + self.chro2len[self.labels[i]]) for i in range(chr_number)]
        # print(self.start_list)
        return stop_list, total_size, chr_number


    def transform_deg(self, ch, pos, total_size):
        return self.to_deg(pos + self.start_list[ch], total_size)

    def transform_pt(self, ch, pos, r, total_size):
        # convert chromosome position to axis coords
        rad = self.to_radian(pos + self.start_list[ch], total_size)
        return r*cos(rad), r*sin(rad)

    def transform_pt1(self,ch, pos, r, total_size):
        # convert chromosome position to axis coords
        # print("transform", ch, pos, r)
    #    print "startlist", self.start_list[ch]
        rad = self.to_radian(pos + self.start_list[ch], total_size)
        return r*cos(rad), r*sin(rad),rad

    def transform_pt2(self, rad, r):
        return r*cos(rad), r*sin(rad)

    def plot_bez_inner(self, p1, p2, cl, total_size):
    #    print "inner"
        a, b, c = p1
        ex1x, ex1y = self.transform_pt(a, b, c, total_size)
        a, b, c = p2
        ex2x, ex2y = self.transform_pt(a, b, c, total_size)
        # Bezier ratio, controls curve, lower ratio => closer to center
        ratio = .5
        x = [ex1x, ex1x*ratio, ex2x*ratio, ex2x]
        y = [ex1y, ex1y*ratio, ex2y*ratio, ex2y]
        step = .01
        t = arange(0, 1+step, step)
        xt = Bezier(x, t)# 贝塞尔曲线
        yt = Bezier(y, t)
        plot(xt, yt, '-', color=cl, lw=.3, alpha=0.3)#alpha 透明度

    def plot_bez_outer(self, p1, p2, cl, total_size):
    #    print "outer"
        a, b, c = p1
        ex1x, ex1y = self.transform_pt(a, b, c, total_size)
        a, b, c = p2
        ex2x, ex2y = self.transform_pt(a, b, c, total_size)
        # Bezier ratio, controls curve, lower ratio => closer to center
        ratio = 1.1
        x = [ex1x, ex1x*ratio, ex2x*ratio, ex2x]
        y = [ex1y, ex1y*ratio, ex2y*ratio, ex2y]
        step = .01
        t = arange(0, 1+step, step)
        xt = Bezier(x, t)
        yt = Bezier(y, t)
        plot(xt, yt, '-', color=cl , lw=.1)

    def plot_bez_Ks(self, rad1, r1, rad2, r2, col):
    #    print "bez Ks 1"
        ex1x, ex1y = self.transform_pt2(rad1, r1)
        ex2x, ex2y = self.transform_pt2(rad2, r2)
        ratio = 0.5#0.5
        x = [ex1x, ex1x*ratio, ex2x*ratio, ex2x]
        y = [ex1y, ex1y*ratio, ex2y*ratio, ex2y]
        step = .01
        t = arange(0, 1+step, step)
        xt = Bezier(x, t)
        yt = Bezier(y, t)
        plot(xt, yt, '-', color=col, lw=.1)

    def plot_bez_Ks2(self, rad1, r1, rad2, r2, col, ratio):
    #    print "bez Ks 2"
        # print('rad1, r1',rad1, r1)
        ex1x, ex1y = self.transform_pt2(rad1, r1)
        ex2x, ex2y = self.transform_pt2(rad2, r2)
        # ratio = -0.7#0.5
        sita = pi / 2
        if ex1x != ex2x:
            sita = atan((ex2y-ex1y)/(ex2x-ex1x))
        d = sqrt((ex2x-ex1x)**2+(ex2y-ex1y)**2)
        L = d * ratio
        P1x = ex1x + L*sin(sita)
        P1y = ex1y - L*cos(sita)
        P2x = ex2x + L*sin(sita)
        P2y = ex2y - L*cos(sita)
        step = .01
        t = arange(0, 1+step, step)
        x=[ex1x, P1x, P2x, ex2x]
        y=[ex1y, P1y, P2y, ex2y]
        # print('x,y,t',x,y,t)
        xt = Bezier(x,t)
        yt = Bezier(y,t)
        plot(xt, yt, '-', color = col, lw = 0.3)#0.1

    def plot_sector(self, s, root, total_size,colors):
        block_id, chr_id, start_bp, stop_bp = s
        theta0, theta1 = self.transform_deg(chr_id, start_bp, total_size), self.transform_deg(chr_id, stop_bp, total_size)
        dtheta = .1
        p1 = Wedge((0, 0), self.radius_a, theta0, theta1, dtheta=dtheta, fc="w", ec="w")
        p2 = Wedge((0, 0), self.radius_b, theta0, theta1, dtheta=dtheta, fc=colors[block_id], ec="w")
        root.add_patch(p2)
        root.add_patch(p1)

    def plot_scale(self,length,number):
        plot([0.5-length-0.05,0.5-0.05],[-0.45,-0.45], color='#333631',linestyle='-', lw = 1.5)
        plot([0.5-length-0.05,0.5-length-0.05],[-0.45,-0.44], color='red',linestyle='-', lw = 1)
        plot([0.5-0.05,0.5-0.05],[-0.45,-0.44], color='#333631',linestyle='-', lw = 1)
        text(0.5-length-0.05+(length/2)-0.02,-0.43, s = str(number),color='black', fontsize = 8, alpha=0.8)# 添加基因标注

    def classify(self, Ks, start1, start2, grade):
        colorlist = ['#e83929', '#ffec47', '#cc7eb1', 'chocolate', 'yellow', 'palegreen', 'teal', 'dodgerblue', 'blueviolet', 'fuchsia']
        lt = self.ks_concern.strip('\n').split(',')
        length = len(lt)
        if (Ks > eval(lt[-1]) or Ks < eval(lt[0])):
            color = 'gray'
            class1 = grade
            if (eval(self.peripheral) == 'True'):
                return color, class1
            else:
                return None,None
        else:
            color = colorlist[grade]
            class1 = grade
            return color, class1

    def run(self):
        self.radius_a =  float(self.radius)
        self.radius_b = self.radius_a + 0.005#.33, .335   # 半径控制参数 
        self.sm_radius=(self.radius_b-self.radius_a)/2 #telomere capping
        if (os.path.exists(self.savecsv)):
            os.remove(self.savecsv)
        lt = ['gene1', 'gene2', 'start1', 'shift1', 'start2', 'shift2', 'class1']
        with open(self.savecsv, "a", newline='', encoding='utf-8') as file:
            writer = csv.writer(file ,delimiter=',')
            writer.writerow(lt)
        root = self.ksrun()
        stop_list, total_size, chr_number = self.zj()
        # alphagenepairs = open(self.genefamily, 'r', encoding='utf-8')
        ## sort gene according to lacation on circle
        genessorted = []
        geneno = 0
        for i in range(len(self.genes)):
            if geneno == 0:
                genessorted.append(self.genes[0])
                geneno = geneno + 1
            else:
                firstgene = genessorted[0]
                lastgene = genessorted[-1]
                chf = firstgene.split("^")[0]# 第一个基因的染色体
                chl = lastgene.split("^")[0]# 最后一个基因的染色体
                #print firstgene, lastgene, chf, chl, self.gene2pos[firstgene]
                posf = self.gene2pos[firstgene] + self.start_list[self.labels.index(chf)]
                posl = self.gene2pos[lastgene] + self.start_list[self.labels.index(chl)]
                chi = self.genes[i].split("^")[0]
                # print(chi)
                if (chi not in self.labels):
                    continue
                posi = self.gene2pos[self.genes[i]] + self.start_list[self.labels.index(chi)]
        #        print posf, posl, posi
                if posi <= posf:
                    genessorted[0:0] = [self.genes[i]]
                elif posi >= posl:
                    # print(self.genes[i])
                    genessorted.append(self.genes[i])
                else:
                    for j in range(len(genessorted)-1):
                        chj = genessorted[j].split("^")[0]
                        posj = self.gene2pos[genessorted[j]] + self.start_list[self.labels.index(chj)]
                        chj1 = genessorted[j+1].split("^")[0]
                        posj1 = self.gene2pos[genessorted[j+1]]+self.start_list[self.labels.index(chj1)]
                        #print posj, posj1, posi
                        if posi > posj and posi < posj1:
                            genessorted[j+1:j+1] = [self.genes[i]]
        #    print "genesort ", genessorted
        ##input genepairs inter sircle
        ###########
        ###########alpha genepairs
        ###########
        rowno = 0
        istoread = 0
        genepairs = self.genepair2Ks
        for pair in genepairs.keys():
            id1, id2 = pair.split()[0],pair.split()[1]
            pos1 = self.gene2pos[id1]
            pos2 = self.gene2pos[id2]
            chro1 = id1.split("^")[0]
            chro2 = id2.split("^")[0]
            # print(chro1,chro2)
            if(id1 not in genessorted or id2 not in genessorted):
                continue
            order1 = self.labels.index(chro1)
            order2 = self.labels.index(chro2)
            # print(order1, pos1, self.radius_a,order2, pos2, self.radius_a, "green", total_size)
            self.plot_bez_inner((order1, pos1, self.radius_a*0.95), (order2, pos2, self.radius_a*0.95), "#00a381", total_size)#绘制内部共线性
            rowno = rowno + 1
        # the chromosome layout
        j = 0
        for start, stop in zip(self.start_list, stop_list):# 绘制染色体
            start, stop = self.to_radian(start, total_size), self.to_radian(stop, total_size)
            # shaft
            self.plot_arc(start, stop, self.radius_a)
            self.plot_arc(start, stop, self.radius_b)
            # telemere capping
            clockwise=False
            self.plot_cap(start, clockwise)
            clockwise=True
            # print(start)
            self.plot_cap(stop, clockwise)
            label_x, label_y = self.rad_to_coord((start+stop)/2, self.radius_b*0.9)#1.2
            #print label_x, label_y
            text(label_x, label_y, self.labels[j], horizontalalignment="center", verticalalignment="center", fontsize = 7, color = 'black')
            j+=1
        ########
        # print('#####################################')
        ########
        ### define shift level and draw gene blocks

        gene_average = gene_length(self.gff)
        ### Again: define shift level and draw gene blocks
        shiftlevel = 0
        gene2shift = {}
        gene2location = {}
        cho = genessorted[0].split("^")[0]
        pos0 = self.gene2pos[genessorted[0]] + self.start_list[self.labels.index(cho)]
        start = self.to_radian(pos0, total_size)
        # print('4')
        laststart = start
        # self.genepair0.append(genessorted[0])# 
        # print('2')
        shiftlist = []
        space_ = 0
        for i in range(len(genessorted)):## 单独循环
            chi = genessorted[i].split("^")[0]
            posi = self.gene2pos[genessorted[i]] + self.start_list[self.labels.index(chi)]
            # chain = self.gene2chain[genessorted[i]]
            start = self.to_radian(posi, total_size)
            # print('start',start)
            if(start-laststart<float(self.space)):#堆叠 前一个基因和后一个基因之间的弧度差小于blocklenngth
                # print('start-laststart<self.space',start-laststart,self.space)
                shiftlevel = shiftlevel + 1
                space_ = space_ + (start-laststart)
                shiftlist.append(space_)
            else:
                shiftlevel=0
                space_ = 0
            self.plot_arc_block(start, self.radius_a - (shiftlevel+1) * self.shiftratio * self.blockthick)# 主要的块儿来源
            laststart = start
        gene_size_ = max(shiftlist)

        shiftlevel = 0
        gene2shift = {}
        gene2location = {}
        cho = genessorted[0].split("^")[0]
        # print(genessorted[0])
        pos0 = self.gene2pos[genessorted[0]] + self.start_list[self.labels.index(cho)]
        start = self.to_radian(pos0, total_size)
        # print(start, self.radius_a - shiftlevel * self.shiftratio * self.blockthick)
        self.plot_arc_block1(start, self.radius_a - shiftlevel * self.shiftratio * self.blockthick,gene_size_)#第一个基因块儿
        gene2location[genessorted[0]]= start
        gene2shift[genessorted[0]] = shiftlevel
        laststart = start
        # print(genessorted)
        for i in range(len(genessorted)):
            chi = genessorted[i].split("^")[0]
            posi = self.gene2pos[genessorted[i]] + self.start_list[self.labels.index(chi)]
            # chain = self.gene2chain[genessorted[i]]
            start = self.to_radian(posi, total_size)
            if(start-laststart < float(self.space)):
                shiftlevel = shiftlevel + 1
            else:
                shiftlevel=0
            gene2location[genessorted[i]] = start
            gene2shift[genessorted[i]] = shiftlevel
            laststart = start
        ### draw Ks lines between genes
        #### draw tick showing scale of chromosomes
        if self.file_type == 'famCircle':
            pair_list = read_famCircle(self.pair_file,self.species)
        elif self.file_type == 'coliearity':
            pair_list = read_coliearity0(self.pair_file,self.species)
        elif self.file_type == 'BLAST':
            pair_list = read_blast(self.pair_file,self.species)
        else:
            print("Exception string: File is of unknown type")
            exit()
        # print(self.genepair2Ks.keys())
        for genepair in pair_list:
            
            if genepair not in self.genepair2Ks.keys():
                continue
            # print(genepair)
        # for genepair in self.genepair2Ks.keys():
            gene1, gene2 = genepair.split(' ')
            # print(gene1, gene2)
            Ks = self.genepair2Ks[genepair]
            # print(Ks)
            if gene1 not in gene2location.keys() or gene2 not in gene2location:
                continue# 此处还要修改

            start1 = gene2location[gene1]
            shift1 = gene2shift[gene1]
            start2 = gene2location[gene2]
            shift2 = gene2shift[gene2]
            if abs(start1 - start2) > gene_size_:
                continue
            # print(Ks)
            a=self.gene2pos[gene1]
            b=self.gene2pos[gene2]
            # print('a,b:',a,b) #  a,b: 45618718 48586752 基因起始位置
            # pitch = abs(b-a)
            space_ks = (abs(float(self.ks_concern.strip('\n').split(',')[1]) - float(self.ks_concern.strip('\n').split(',')[0])))/3
            grade = int(Ks / space_ks)# 分级
            color, class1 = self.classify(Ks, start1, start2, grade)
            # print(color)
            color = return_col(Ks,float(self.ks_concern.split(',')[0]),float(self.ks_concern.split(',')[1]))
            ratio = 0.5
            if (color != None):
                if (self.clusters == "True"):
                    lt = [gene1, gene2, start1, shift1, start2, shift2, class1]
                    with open(self.savecsv, "a", newline='', encoding='utf-8') as file:
                        writer = csv.writer(file ,delimiter=',')
                        writer.writerow(lt)
                    self.plot_bez_Ks2(start1, self.radius_a - (shift1+1) * self.shiftratio * self.blockthick, start2, self.radius_a - (shift2+1) * self.shiftratio * self.blockthick, color, ratio)
                else:
                    self.plot_bez_Ks2(start1, self.radius_a - (shift1+1) * self.shiftratio * self.blockthick, start2, self.radius_a - (shift2+1) * self.shiftratio * self.blockthick, 'red', ratio)
                    lt = [gene1, gene2, start1, shift1, start2, shift2, class1]
                    with open(self.savecsv, "a", newline='', encoding='utf-8') as file:
                        writer = csv.writer(file ,delimiter=',')
                        writer.writerow(lt)
            else :
                # print('间距不达标')
                pass
        order_length = gene_length_order(self.lens)
        angle2 = 0
        angle0,angle1 = 0,0
        for i in range(chr_number):
            pos = 0
            while pos < self.orderlist[self.labels[i]]:
                if pos == 0 :
                    color = "red"
                else:
                    color = 'black'
                index_order = pos * order_length
                xx1, yy1 = self.transform_pt(i, int(index_order), self.radius_b-0.001, total_size)
                xx2, yy2 = self.transform_pt(i, int(index_order), self.radius_b-0.004, total_size)
                if pos == 1500:
                    xx3, yy3, angle0 = self.transform_pt1(i, int(index_order), self.radius_b-0.003, total_size)
                elif pos == 3000:
                    xx3, yy3, angle1 = self.transform_pt1(i, int(index_order), self.radius_b-0.003, total_size)
                    angle2 = abs(angle0 - angle1)
                # plot([xx1, xx2], [yy1, yy2], "k-", lw = .2)
                # pos = pos + 10000000
                # print()
                # xx3, yy3, angle0 = self.transform_pt1(i, int(index_order), self.radius_b-0.003, total_size)
                # if xx3 > 0:
                #     xx3 = xx3 - 0.008
                #     yy3 = yy3 - 0.008
                # if xx3 < 0:
                #     xx3 = xx3 - 0.008
                #     yy3 = yy3 - 0.008
                # # print('xx3, yy3, angle0, pi/2 - angle0',xx3, yy3, (pi/2)-angle0)
                # angle1 = ((angle0 - (pi/2))/(2*pi))*360
                plot([xx1, xx2], [yy1, yy2], color=color,linestyle='-', lw = .5)
                # text(xx3,yy3, s = str(pos),color='b', fontsize = 5, rotation=angle1, alpha=0.8)# 添加基因标注
                pos = pos + 1500
        self.plot_scale(angle2*self.radius_b,1500)
        # print('3')
        root.set_xlim(-.8, .8)#-.5, .5
        root.set_ylim(-.8, .8)
        root.set_axis_off()
        savefig(self.savefile, dpi=1000)
        sys.exit(0)



# -*- encoding: utf-8 -*-
'''
@File        :line.py
@Time        :2021/07/11 12:44:00
@Author        :charles kiko
@Version        :1.0
@Contact        :charles_kiko@163.com
@Desc        :共线性局部
'''

import csv
import sys
import re
from math import *
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
from matplotlib.patches import *
from matplotlib.patches import Circle, Ellipse, Arc
from pylab import *
from famCircle.bez import *

class line():
    def __init__(self, options):
        self.makers = 200
        class1 = False
        for k, v in options:
            setattr(self, str(k), v)
            print(k, ' = ', v)

    def lower_(self,name):
        name = name.lower()
        return name

    def readgff_(self,file):
        myList = []
        for line in open(file,'r'):
            lt = line.strip('\n').split()
            lt[0],lt[1] = lt[1],lt[0]
            lt[0] = self.lower_(lt[0])
            myList.append(lt)
        matrix_gff0 = pd.DataFrame(data = myList)
        matrix_gff0 = matrix_gff0.set_index(0)
        return matrix_gff0

    def read_csv(self):
        self.chrolist = [self.chr1_name,self.chr2_name]
        gff1 = read_gff(self.gff1)
        gff2 = read_gff(self.gff2)
        gff = dict( gff1, **gff2 )
        print(gff)
        matrix_chr1 = pd.read_csv(self.lens1, sep='\t', header=None, index_col=0)
        length1 = matrix_chr1.loc[self.chr1_name,2]
        matrix_chr2 = pd.read_csv(self.lens2, sep='\t', header=None, index_col=0)
        length2 = matrix_chr2.loc[self.chr2_name,2]
        if length1 >= length2:
            x1 = 0.8
            x2 = length2*(0.8/length1)
            maker_str = True
        else :
            x1 = 0.8
            x2 = length1*(0.8/length2)
            maker_str = False
        maker1 ,maker2 = int(max(length2,length1)/self.makers), int(min(length2,length1)/self.makers)
        chro1 = self.chr1_name
        chro2 = self.chr2_name
        matrix_gff1 = self.readgff_(self.gff1)
        matrix_gff2 = self.readgff_(self.gff2)
        matrix_gff1.groupby(by=0)
        matrix_gff2.groupby(by=0)
        return x1,x2,maker1,maker2,matrix_gff1,matrix_gff2,chro1,chro2,maker_str,length1,length2

    def read_colinearity(self,file):
        pair = []
        pair_list = []
        if self.pairfile_type == 'famCircle':
            with open(file) as f:
                for line in f.readlines():
                    line = line.strip()
                    if  line[0] == "\n":
                        continue
                    elif line[0] == "#":
                        if len(pair_list) != 0:
                            # print(pair_list)#输出
                            pair.append(pair_list)
                        pair_list = []
                        block_dict = {}
                        lt = line.split()
                        block_number = str(lt[2])[:-1]
                        # print(lt)
                        for i in lt:
                            if "=" in i:
                                lt0 = i.split('=')
                                block_dict[str(lt0[0])] = str(lt0[1])
                        if int(block_dict['N']) > int(self.block):
                            class1 = True
                        else:
                            class1 = False
                    else:
                        if class1:
                            lt = line.split()
                            name1 = readname(str(lt[0]),self.chr1_name+self.chr2_name)
                            name2 = readname(str(lt[1]),self.chr1_name+self.chr2_name)
                            lt0 = [name1,name2]
                            # print(lt0)
                            pair_list.append(lt0)
                if len(pair_list) != 0 and class1:
                    # print(pair_list)#输出
                    pair.append(pair_list)
                # print(pair)
                return pair
        elif self.pairfile_type == 'WGDI':
            with open(file) as f:
                for line in f.readlines():
                    line = line.strip()
                    if  line[0] == "\n":
                        continue
                    elif line[0] == "#":
                        if len(pair_list) != 0:
                            # print(pair_list)#输出
                            pair.append(pair_list)
                        pair_list = []
                        block_dict = {}
                        lt = line.split()
                        block_number = str(lt[2])[:-1]
                        # print(lt)
                        for i in lt:
                            if "=" in i:
                                lt0 = i.split('=')
                                block_dict[str(lt0[0])] = str(lt0[1])
                        if int(block_dict['N']) > int(self.block):
                            class1 = True
                        else:
                            class1 = False
                    else:
                        if class1:
                            lt = line.split()
                            name1 = readname(str(lt[0]),self.chr1_name+self.chr2_name)
                            name2 = readname(str(lt[2]),self.chr1_name+self.chr2_name)
                            lt0 = [name1,name2]
                            # print(lt0)
                            pair_list.append(lt0)
                if len(pair_list) != 0 and class1:
                    # print(pair_list)#输出
                    pair.append(pair_list)
                # print(pair)
                return pair
        elif self.pairfile_type == 'MCScanX':
            with open(file) as f:
                for line in f.readlines():
                    line = line.strip()
                    if line[0] == "\n":
                        continue
                    elif line[:12] == '## Alignment':
                        # print(line)
                        if len(pair_list) != 0:
                            pair.append(pair_list)
                        pair_list = []
                        block_dict = {}
                        lt = line.split()
                        block_number = str(lt[2])[:-1]
                        # print(lt)
                        for i in lt:
                            if "=" in i:
                                lt0 = i.split('=')
                                block_dict[str(lt0[0])] = str(lt0[1])
                        if int(block_dict['N']) > int(self.block):
                            class1 = True
                        else:
                            class1 = False
                    else:
                        if class1:
                            lt = line.split()
                            if len(lt) == 5:
                                id1, id2 = str(lt[2]),str(lt[3])
                            elif len(lt) == 4:
                                id1, id2 = str(lt[1]),str(lt[2])
                            elif len(lt) == 6:
                                id1, id2 = str(lt[3]),str(lt[4])
                            else:
                                print(line)
                                print('Parse error!')
                                exit()
                            name1 = readname(id1,self.chr1_name+self.chr2_name)
                            name2 = readname(id2,self.chr1_name+self.chr2_name)
                            lt0 = [name1,name2]
                            pair_list.append(lt0)
                if len(pair_list) != 0 and class1:
                    pair.append(pair_list)
                return pair

        if self.pairfile_type == 'ColinearScan':
            with open(file) as f:
                for line in f.readlines():
                    line = line.strip()
                    # print(line)
                    if line[0] == "+" or line[0] == "\n" or line[0] == ">":
                        continue
                    elif line[:3] == "the":
                        if len(pair_list) != 0:
                            pair.append(pair_list)
                        pair_list = []
                        block_dict = {}
                        lt = line.split()
                        block_number = str(lt[1])[:-2]
                        # print(lt)
                        for i in lt:
                            if "=" in i:
                                lt0 = i.split('=')
                                block_dict[str(lt0[0])] = str(lt0[1])
                        if int(lt[-1]) > int(self.block):
                            class1 = True
                        else:
                            class1 = False
                    else:
                        if class1:
                            lt = line.split()
                            name1 = readname(str(lt[0]),self.chr1_name+self.chr2_name)
                            name2 = readname(str(lt[2]),self.chr1_name+self.chr2_name)
                            lt0 = [name1,name2]
                            # print(lt0)
                            pair_list.append(lt0)
                if len(pair_list) != 0 and class1:
                    # print(pair_list)#输出
                    pair.append(pair_list)
                # print(pair)
                return pair
        elif self.pairfile_type == 'BLAST':
            num = 0
            name_list = []
            with open(file) as f:
                for line in f.readlines():
                    line = line.strip()
                    if  line[0] == "\n":
                        continue
                    lt = line.split()
                    if lt[0] == lt[1]:
                        continue
                    if str(lt[0]) not in name_list:
                        name_list.append(str(lt[0]))
                        num = 1
                    else:
                        if num <= int(self.block):
                            name1 = readname(str(lt[0]),self.chr1_name+self.chr2_name)
                            name2 = readname(str(lt[1]),self.chr1_name+self.chr2_name)
                            lt0 = [name1,name2]
                            pair.append(lt0)
                        else:
                            continue
                return pair
        else:
            print('genepairsfile_type error: File Format not recognized!')
            exit()

    def plot_bez_inner(self, ex1x, ex1y, ex2x, ex2y):
        x = [ex1x, ex1x+((ex2x - ex1x)/3+0.1), ex2x-((ex2x - ex1x)/3+0.1), ex2x]
        y = [ex1y, ex1y+((ex2y - ex1y)/3), ex2y-((ex2y - ex1y)/3), ex2y]
        # ratiox = 0.5
        # ratioy = 1
        # x = [ex1x, ex1x*ratiox, ex2x*ratiox, ex2x]
        # y = [ex1y, ex1y*ratioy, ex2y*ratioy, ex2y]
        # x = [ex1x, ex1x+((ex2x - ex1x)/3), ex2x-((ex2x - ex1x)/3), ex2x]
        # y = [ex1y, ex1y+((ex2y - ex1y)/3), ex2y-((ex2y - ex1y)/3), ex2y]
        step = .01
        t = arange(0, 1+step, step)
        xt = self.Bezier(x, t)# 贝塞尔曲线
        yt = self.Bezier(y, t)
        plot(xt, yt, '-', color='#7ebea5', lw=1, alpha=0.3)#alpha 透明度

    def calculate_coef(self,p0, p1, p2, p3):
        c = 3*(p1 - p0)
        b = 3*(p2 - p1) -c
        a = p3 - p0 - c - b
        return c, b, a
    def Bezier(self,plist, t):
        # p0 : origin, p1, p2 :control, p3: destination
        p0, p1, p2, p3 = plist
        # calculates the coefficient values
        c, b, a = self.calculate_coef(p0, p1, p2, p3)
        tsquared = t**2
        tcubic = tsquared*t
        return a*tcubic + b*tsquared + c*t + p0

    def make_plot(self,maker1,maker2,matrix_gff1,matrix_gff2,pair,chro1,chro2,x1,x2,maker_str,length1,length2):
        fig1 = plt.figure(num=1, figsize=(10, 10))  # 确保正方形在屏幕上显示一致，固定figure的长宽相等
        axes1 = fig1.add_subplot(1, 1, 1)
        plt.xlim((0, 1))
        plt.ylim((0, 1))
        self.map1(axes1,maker1,0.1,0.3,x1,'#ffec47')# 长染色体0.8
        self.map1(axes1,maker2,0.1+((0.8-x2)/2),0.65,x2,'#ffec47')# 短染色体
        if maker_str:
            plt.text(0.45, 0.28, chro1)
            plt.text(0.45, 0.67, chro2)
        else:
            plt.text(0.45, 0.28, chro2)
            plt.text(0.45, 0.67, chro1)
        plt.axis('off')
        self.pair_index(axes1,matrix_gff1,matrix_gff2,pair,chro1,chro2,x1,x2,maker_str,length1,length2)
        plt.savefig(self.savefile,dpi=1000)

    def map1(self,axes1,maker,x,y,x1,color):
        # makers = 15 #标尺
        # x = 0.1 #起始坐标x
        # y = 0.1 #起始坐标y
        # x1 = 0.8#染色体长度
        # color='k'
        lw = 1  #比例线宽
        y1 = 0.001
        w = 0.01
        alpha = 1
        # print(y+w+y1, x, x+x1,y, x, x+x1, lw)
        plt.axhline(y=y, xmin=x, xmax=x+x1, lw=lw, c=color, alpha=alpha)
        plt.axhline(y=y+w+y1, xmin=x, xmax=x+x1, lw=lw, c=color, alpha=alpha)
        for i in range(maker):
            mx = x + ((x1/maker)*i)
            plt.axvline(x=mx, ymin=y, ymax=y+(w/2.5), lw=lw, c=color, alpha=alpha)
        base1 = Arc(xy=(x, y+((w+y1)/2)),    # 椭圆中心，（圆弧是椭圆的一部分而已）
                width=w+y1,    # 长半轴
                height=w+y1,    # 短半轴
                angle=90,    # 椭圆旋转角度（逆时针） 
                theta1=0,    # 圆弧的起点处角度
                theta2=180,    # 圆度的终点处角度
                color=color,
                alpha=alpha,
                linewidth=lw
                )
        base2 = Arc(xy=(x1+x, y+((w+y1)/2)),    # 椭圆中心，（圆弧是椭圆的一部分而已）
                width=w+y1,    # 长半轴
                height=w+y1,    # 短半轴
                angle=-90,    # 椭圆旋转角度（逆时针） 
                theta1=0,    # 圆弧的起点处角度
                theta2=180,    # 圆度的终点处角度
                color=color,
                alpha=alpha,
                linewidth=lw   #线宽像素
                )
        axes1.add_patch(base1)
        axes1.add_patch(base2)
        
    def pair_index(self,axes1,matrix_gff1,matrix_gff2,pair,chro1,chro2,x1,x2,maker_str,length1,length2):
        pairs = []
        for i in pair:
            for j in i:
                if str(str(j[0]).split('^')[0]) == self.chr1_name and str(str(j[1]).split('^')[0]) == self.chr2_name:
                    name11 = str(j[0]).replace("^", "g")
                    name22 = str(j[1]).replace("^", "g")
                    index1 = float(matrix_gff1.loc[name11,5])
                    index2 = float(matrix_gff2.loc[name22,5])
                    # print(index1,index2,'index')
                    if maker_str:
                        prop1 = x1/length1
                        prop2 = x2/length2
                        # print(length1,length2)
                        # pair_lt = [[0.1+index1*prop1,0.3],[0.1+((0.8-x2)/2)+index2*prop2,0.65]]
                        # pairs.append(pair_lt)
                        self.plot_bez_inner(0.1+((0.8-x2)/2)+index2*prop2,0.65-0.004,0.1+index1*prop1,0.3+0.01+0.004)
                        # plt.plot([0.1+((0.8-x2)/2)+index2*prop2,0.1+index1*prop1],[0.65-0.004,0.3+0.01+0.004],c='g')
                    else:
                        prop1 = x2/length1
                        prop2 = x1/length2
                        # pair_lt = [[0.1+index2*prop2,0.3],[0.1+((0.8-x2)/2)+index1*prop1,0.65]]
                        # pairs.append(pair_lt)

                        self.plot_bez_inner(0.1+((0.8-x2)/2)+index1*prop1,0.65-0.004,0.1+index2*prop2,0.3+0.01+0.004)
                        # plt.plot([0.1+((0.8-x2)/2)+index1*prop1,0.1+index2*prop2],[0.65-0.004,0.3+0.01+0.004],c='r')
    def run(self):
        x1,x2,maker1,maker2,matrix_gff1,matrix_gff2,chro1,chro2,maker_str,length1,length2 = self.read_csv()
        pair = self.read_colinearity(self.pairs_file)
        self.make_plot(maker1,maker2,matrix_gff1,matrix_gff2,pair,chro1,chro2,x1,x2,maker_str,length1,length2)
# iclmap0 = iclmap()
# x1,x2,maker1,maker2,matrix_gff1,matrix_gff2,chro1,chro2,maker_str,length1,length2 = iclmap0.read_csv()
# pair = iclmap0.read_colinearscan('ssu_mes.collinearity')
# iclmap0.make_plot(maker1,maker2,matrix_gff1,matrix_gff2,pair,chro1,chro2,x1,x2,maker_str,length1,length2)



# -*- encoding: utf-8 -*-
'''
@File        :typing.py
@Time        :2021/09/28 09:03:24
@Author        :charles kiko
@Version        :1.0
@Contact        :charles_kiko@163.com
@Desc        :修改格式
'''


import os
import numpy as np
import pandas as pd

class typing():
    def __init__(self, options):
        for k, v in options:
            setattr(self, str(k), v)
            print(k, ' = ', v)

    def readoutfile(self):
        out_list = str(self.domainlist).split(',')
        fam_out = []
        for i in out_list:
            pathlist = self.domainpath + '/' + str(i)
            f = open(pathlist, 'r', encoding='utf-8')
            for row in f:
                if row[0] != '#' and row[0] != '\n':
                    row = row.strip('\n').split()
                    fam_out.append(row)
        df = pd.DataFrame(fam_out)
        df.rename(columns={0: 'target_name', 1: 'accession',
                                2: 'tlen', 3: 'query_name', 4: 'accession1', 5: 'qlen',
                                6: 'E-value', 7: 'score', 8: 'bias', 9: '#', 10: 'of', 11: 'c-Evalue',
                                12: 'i-Evalue', 13: 'score2', 14: 'bias2', 15: 'from', 16: 'to',
                                17: 'from1', 18: 'to1', 19: 'from2', 20: 'to2', 21: 'acc',
                                22: 'description_of_target'}, inplace=True)
        df.drop_duplicates(subset = 'target_name')# 去重
        return df

    def run(self):
        df = self.readoutfile()
        fam_name = np.array(df['target_name'].astype(str))
        for i in fam_name:
            with open(self.savefile, "a", newline='', encoding='utf-8') as file:
                file.write(i + '\n')
                file.close()


# -*- encoding: utf-8 -*-
'''
@File        :Ks_block.py
@Time        :2021/09/28 11:20:59
@Author        :charles kiko
@Version        :1.0
@Contact        :charles_kiko@163.com
@Desc        :block的ks
'''


from scipy.stats.kde import gaussian_kde
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
from famCircle.bez import *
import sys
import csv


class Ks_block():
    def __init__(self, options):
        self.vertical = "False"
        self.model = "NG86"
        for k, v in options:
            setattr(self, str(k), v)
            print(k, ' = ', v)

    def readks(self):
        read_ks = {}
        f = open(self.ks, 'r', encoding='utf-8')
        for row in f:
            if row[0] != '#' and row[0] != '\n':
                row = row.strip('\n').split('\t')
                if row[0] != 'id1' and len(row) != 2:
                    pair = str(row[0]) + '_' + str(row[1])
                    lt = row[2:]
                    read_ks[pair] = lt
        # print(read_ks)
        return read_ks

    def readblock(self,read_ks):
        model = self.model
        lt0 = ['number', 'id1', 'id2', 'start1','end1', 'start2', 'end2','length','density1','density2','median','average','score','e_value']
        with open(self.savecsv, "a", newline='', encoding='utf-8') as file:
            writer = csv.writer(file ,delimiter=',')
            writer.writerow(lt0)
        num = 0
        if self.blockfile_type == 'famCircle':
            f = open(self.blockfile, 'r', encoding='utf-8')
            id1s,id2s = [],[]
            index1,index2 = [],[]
            score,pvalue = 0,0
            for row in f:
                if row[0] != '#' and row[0] != '\n':
                    row = row.strip('\n').split()
                    id10, id20 = str(row[0]),str(row[1])
                    # print(id10)
                    id1s.append(id10)
                    id2s.append(id20)
                    # print(readname(id10,self.species_list))
                    index1.append(int(readname(id10,self.species_list).split('^')[1]))
                    index2.append(int(readname(id20,self.species_list).split('^')[1]))
                elif (row[0] == '#'):
                    # print(row)
                    if len(id1s) == 0:
                        id1s,id2s = [],[]
                        index1,index2 = [],[]
                        doc = {}
                        row = row.strip('\n').split()
                        for i in row:
                            if '=' in i:
                                lt = i.split('=')
                                doc[str(lt[0])] = str(lt[1])
                            else:
                                continue
                        score,pvalue = float('inf'),float(1)
                    else:
                        num += 1
                        ks_list = []
                        for i in range(len(id1s)):
                            id1 = str(id1s[i])
                            id2 = str(id2s[i])
                            try:
                                id_ks = read_ks[id1 + '_' + id2]
                            except:
                                try:
                                    id_ks = read_ks[id2 + '_' + id1]
                                except:
                                    # print ("not found " + id1 + '_' + id2)
                                    continue
                            # print(id_ks)
                            if model == 'NG86':
                                ks0 = id_ks[1]
                            elif model == 'YN00':
                                ks0 = id_ks[3]
                            if eval(ks0) >= 10:
                                continue
                            ks_list.append(eval(ks0))
                        # print(ks_list)
                        if ks_list == []:
                            continue
                        ks_list0 = sorted(ks_list)
                        median = np.median(ks_list0)
                        # print(len(ks_list0))
                        average = sum(ks_list0)/float(len(ks_list0))
                        density1 = len(index1)/float(abs(index1[-1] - index1[0]))
                        density2 = len(index2)/float(abs(index2[-1] - index2[0]))
                        print(id1,id2,len(index2),pvalue)
                        # if  len(index2) < eval(self.blocklength) or pvalue < eval(self.e_value):
                        if  len(index2) < eval(self.blocklength):
                            # print(lt)
                            pass
                        else:
                            # print(lt)
                            lt = [num,id1,id2,int(index1[0]),int(index1[-1]),int(index2[0]),int(index2[-1]),len(ks_list0),density1,density2,median,average,score,pvalue]
                            print(lt)
                            with open(self.savecsv, "a", newline='', encoding='utf-8') as file:
                                writer = csv.writer(file ,delimiter=',')
                                writer.writerow(lt)
                        id1s,id2s = [],[]
                        index1,index2 = [],[]
                        doc = {}
                        row = row.strip('\n').split()
                        for i in row:
                            if '=' in i:
                                lt = i.split('=')
                                doc[str(lt[0])] = str(lt[1])
                            else:
                                continue
                        score,pvalue = float('inf'),float(0)
        elif self.blockfile_type == 'WGDI':
            f = open(self.blockfile, 'r', encoding='utf-8')
            id1s,id2s = [],[]
            index1,index2 = [],[]
            score,pvalue = 0,0
            for row in f:
                if row[0] != '#' and row[0] != '\n':
                    row = row.strip('\n').split()
                    id1s.append(str(row[0]))
                    id2s.append(str(row[2]))
                    index1.append(eval(row[1]))
                    index2.append(eval(row[3]))
                elif (row[0] == '#'):
                    if len(id1s) == 0:
                        id1s,id2s = [],[]
                        index1,index2 = [],[]
                        doc = {}
                        row = row.strip('\n').split()
                        for i in row:
                            if '=' in i:
                                lt = i.split('=')
                                doc[str(lt[0])] = str(lt[1])
                            else:
                                continue
                        score,pvalue = float(doc['score']),float(doc['pvalue'])
                    else:
                        num += 1
                        ks_list = []
                        for i in range(len(id1s)):
                            id1 = str(id1s[i])
                            id2 = str(id2s[i])
                            try:
                                id_ks = read_ks[id1 + '_' + id2]
                            except:
                                try:
                                    id_ks = read_ks[id2 + '_' + id1]
                                except:
                                    # print ("not found " + id1 + '_' + id2)
                                    continue
                            # print(id_ks)
                            if model == 'NG86':
                                ks0 = id_ks[1]
                            elif model == 'YN00':
                                ks0 = id_ks[3]
                            if eval(ks0) >= 10:
                                continue
                            ks_list.append(eval(ks0))
                        # print(ks_list)
                        if ks_list == []:
                            continue
                        ks_list0 = sorted(ks_list)
                        median = np.median(ks_list0)
                        # print(len(ks_list0))
                        average = sum(ks_list0)/float(len(ks_list0))
                        density1 = len(index1)/float(abs(index1[-1] - index1[0]))
                        density2 = len(index2)/float(abs(index2[-1] - index2[0]))
                        if  len(index2) < eval(self.blocklength) or pvalue < eval(self.e_value):
                            pass
                        else:
                            lt = [num,id1,id2,int(index1[0]),int(index1[-1]),int(index2[0]),int(index2[-1]),len(ks_list0),density1,density2,median,average,score,pvalue]
                            with open(self.savecsv, "a", newline='', encoding='utf-8') as file:
                                writer = csv.writer(file ,delimiter=',')
                                writer.writerow(lt)
                        id1s,id2s = [],[]
                        index1,index2 = [],[]
                        doc = {}
                        row = row.strip('\n').split()
                        for i in row:
                            if '=' in i:
                                lt = i.split('=')
                                doc[str(lt[0])] = str(lt[1])
                            else:
                                continue
                        score,pvalue = float(doc['score']),float(doc['pvalue'])
        elif self.blockfile_type == 'ColinearScan':
            f = open(self.blockfile, 'r', encoding='utf-8')
            id1s,id2s = [],[]
            index1,index2 = [],[]
            score,pvalue = 0,0
            for row in f:
                if row[0] != '+' and row[0] != '\n':
                    row = row.strip('\n').split()
                    id10, id20 = str(row[0]),str(row[2])
                    id1s.append(id10)
                    id2s.append(id20)
                    index1.append(int(readname(id10,self.species_list).split('^')[1]))
                    index2.append(int(readname(id20,self.species_list).split('^')[1]))
                elif (row[0] == '#'):
                    if len(id1s) == 0:
                        id1s,id2s = [],[]
                        index1,index2 = [],[]
                        doc = {}
                        row = row.strip('\n').split()
                        for i in row:
                            if '=' in i:
                                lt = i.split('=')
                                doc[str(lt[0])] = str(lt[1])
                            else:
                                continue
                        score,pvalue = float('inf'),float(0)
                    else:
                        num += 1
                        ks_list = []
                        for i in range(len(id1s)):
                            id1 = str(id1s[i])
                            id2 = str(id2s[i])
                            try:
                                id_ks = read_ks[id1 + '_' + id2]
                            except:
                                try:
                                    id_ks = read_ks[id2 + '_' + id1]
                                except:
                                    # print ("not found " + id1 + '_' + id2)
                                    continue
                            # print(id_ks)
                            if model == 'NG86':
                                ks0 = id_ks[1]
                            elif model == 'YN00':
                                ks0 = id_ks[3]
                            if eval(ks0) >= 10:
                                continue
                            ks_list.append(eval(ks0))
                        # print(ks_list)
                        if ks_list == []:
                            continue
                        ks_list0 = sorted(ks_list)
                        median = np.median(ks_list0)
                        # print(len(ks_list0))
                        average = sum(ks_list0)/float(len(ks_list0))
                        density1 = len(index1)/float(abs(index1[-1] - index1[0]))
                        density2 = len(index2)/float(abs(index2[-1] - index2[0]))
                        if  len(index2) < eval(self.blocklength) or pvalue < eval(self.e_value):
                            pass
                        else:
                            lt = [num,id1,id2,int(index1[0]),int(index1[-1]),int(index2[0]),int(index2[-1]),len(ks_list0),density1,density2,median,average,score,pvalue]
                            with open(self.savecsv, "a", newline='', encoding='utf-8') as file:
                                writer = csv.writer(file ,delimiter=',')
                                writer.writerow(lt)
                        id1s,id2s = [],[]
                        index1,index2 = [],[]
                        doc = {}
                        row = row.strip('\n').split()
                        for i in row:
                            if '=' in i:
                                lt = i.split('=')
                                doc[str(lt[0])] = str(lt[1])
                            else:
                                continue
                        score,pvalue = float('inf'),float(0)
        elif self.blockfile_type == 'MCScanX':
            f = open(self.blockfile, 'r', encoding='utf-8')
            id1s,id2s = [],[]
            index1,index2 = [],[]
            score,pvalue = 0,0
            for row in f:
                if row[0] != '#' and row[0] != '\n':
                    row = row.strip('\n').split()
                    if len(row) == 5:
                        id10, id20 = str(row[2]),str(row[3])
                    elif len(row) == 4:
                        id10, id20 = str(row[1]),str(row[2])
                    elif len(row) == 6:
                        id10, id20 = str(row[3]),str(row[4])
                    else:
                        print(row)
                        print('Parse error!')
                        exit()
                    id1s.append(id10)
                    id2s.append(id20)
                    index1.append(int(readname(id10,self.species_list).split('^')[1]))
                    index2.append(int(readname(id20,self.species_list).split('^')[1]))
                elif (row[:12] == '## Alignment'):
                    if len(id1s) == 0:
                        id1s,id2s = [],[]
                        index1,index2 = [],[]
                        doc = {}
                        row = row.strip('\n').split()
                        for i in row:
                            if '=' in i:
                                lt = i.split('=')
                                doc[str(lt[0])] = str(lt[1])
                            else:
                                continue
                        score,pvalue = float(doc['score']),float(doc['e_value'])
                    else:
                        num += 1
                        ks_list = []
                        for i in range(len(id1s)):
                            id1 = str(id1s[i])
                            id2 = str(id2s[i])
                            try:
                                id_ks = read_ks[id1 + '_' + id2]
                            except:
                                try:
                                    id_ks = read_ks[id2 + '_' + id1]
                                except:
                                    # print ("not found " + id1 + '_' + id2)
                                    continue
                            # print(id_ks)
                            if model == 'NG86':
                                ks0 = id_ks[1]
                            elif model == 'YN00':
                                ks0 = id_ks[3]
                            if float(ks0) >= 10 or float(ks0) <= 0.0:
                                continue
                            ks_list.append(float(ks0))
                        # print(ks_list)
                        if ks_list == []:
                            continue
                        ks_list0 = sorted(ks_list)
                        median = np.median(ks_list0)
                        # print(len(ks_list0))
                        average = sum(ks_list0)/float(len(ks_list0))
                        density1 = len(index1)/float(abs(index1[-1] - index1[0]))
                        density2 = len(index2)/float(abs(index2[-1] - index2[0]))
                        if  len(index2) < eval(self.blocklength) or pvalue > eval(self.e_value):
                            pass
                        else:
                            lt = [num,id1,id2,int(index1[0]),int(index1[-1]),int(index2[0]),int(index2[-1]),len(ks_list0),density1,density2,median,average,score,pvalue]
                            with open(self.savecsv, "a", newline='', encoding='utf-8') as file:
                                writer = csv.writer(file ,delimiter=',')
                                writer.writerow(lt)
                        id1s,id2s = [],[]
                        index1,index2 = [],[]
                        doc = {}
                        row = row.strip('\n').split()
                        for i in row:
                            if '=' in i:
                                lt = i.split('=')
                                doc[str(lt[0])] = str(lt[1])
                            else:
                                continue
                        score,pvalue = float(doc['score']),float(doc['e_value'])
        else:
            print('blockfile_type error: File Format not recognized!')
            exit()

        if len(id1s) != 0:
            num += 1
            ks_list = []
            for i in range(len(id1s)):
                id1 = str(id1s[i])
                id2 = str(id2s[i])
                # print(id1,id2)
                try:
                    id_ks = read_ks[id1 + '_' + id2]
                except:
                    try:
                        id_ks = read_ks[id2 + '_' + id1]
                    except:
                        print("not found " + id1 + '_' + id2)
                        continue
                if model == 'NG86':
                    ks0 = id_ks[1]
                elif model == 'YN00':
                    ks0 = id_ks[3]
                # print(ks0)
                ks_list.append(eval(ks0))
            ks_list0 = sorted(ks_list)
            median = np.median(ks_list0)
            average = sum(ks_list0)/float(len(ks_list0))
            density1 = len(index1)/float(abs(index1[-1] - index1[0]))
            density2 = len(index2)/float(abs(index2[-1] - index2[0]))
            if  len(index2) < eval(self.blocklength) or pvalue < eval(self.e_value):
                pass
            else:
                print(ks0)
                lt = [num,id1,id2,int(index1[0]),int(index1[-1]),int(index2[0]),int(index2[-1]),len(ks_list0),density1,density2,median,average,score,pvalue]
                with open(self.savecsv, "a", newline='', encoding='utf-8') as file:
                    writer = csv.writer(file ,delimiter=',')
                    writer.writerow(lt)
            id1s,id2s = [],[]
            index1,index2 = [],[]
            score,pvalue = 0,0

    def readcsv(self): 
        data = pd.read_csv(self.savecsv,encoding='utf-8')
        lt = [list(data['median']),list(data['average'])]
        return lt

        # 绘制核密度曲线图
    def KdePlot(self,list0):
        vertical = 'False'
        x = list0[0]# 中位数x = list(map(int, x))
        y = list0[1]# 平均数
        plt.figure(figsize=(20,10),dpi=800)
        plt.grid(c='grey',ls='--',linewidth=0.3)
        # print(y)
        dist_space = np.linspace(float(self.area.split(',')[0]),float(self.area.split(',')[1]), 500)
        # print(y)
        kdemedian = gaussian_kde(y)
        kdemedian.set_bandwidth(bw_method=kdemedian.factor / 3.)
        # plt.fill_between(dist_space, y1=0, y2=kdemedian(dist_space), facecolor='#00a3af', alpha=0.5)
        xx = list(dist_space)[list(kdemedian(dist_space)).index(max(list(kdemedian(dist_space))))]
        xx0 = round(xx,4)
        plt.plot(dist_space, kdemedian(dist_space), color='#00a3af',
                 label='average_KDE_' + str(xx0))

        dist_space = np.linspace(float(self.area.split(',')[0]),float(self.area.split(',')[1]), 500)
        kdemedian = gaussian_kde(x)
        kdemedian.set_bandwidth(bw_method=kdemedian.factor / 3.)
        plt.fill_between(dist_space, y1=0, y2=kdemedian(dist_space), facecolor='#d3381c', alpha=0.5)
        xx = list(dist_space)[list(kdemedian(dist_space)).index(max(list(kdemedian(dist_space))))]
        xx0 = round(xx,4)
        plt.plot(dist_space, kdemedian(dist_space), color='#d3381c',
                 label='median_KED_' + str(xx0))
        plt.legend()
        plt.title(self.species_list + ' block Ks kernel density estimation')# 设置图片标题
        plt.xlabel('ks')# 设置 x 轴标签
        plt.ylabel('density')# 设置 y 轴标签

    def run(self):
        if (os.path.exists(self.savecsv)):
            os.remove(self.savecsv)
        read_ks = self.readks()
        self.readblock(read_ks)
        lt = self.readcsv()
        print(lt)
        self.KdePlot(lt)
        plt.savefig(self.savefile)# 存储图片
        sys.exit(0)
